/**
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package elements;

import frame.VisualScene;

/**
 * Função que faz o papel de um Filtro.
 * @author Luiz Gustavo Ender
 */
public class Filter {

    private String proto, dIP, sIP, dport, sport;
    private Objeto dest;
    private VisualScene scene;

    public Filter(String proto, String dport, String dIP, String sport, String sIP, VisualScene scene) {
        this.proto = proto;
        this.dport = dport;
        this.dIP = dIP;
        this.sport = sport;
        this.sIP = sIP;
        this.scene = scene;
    }

    public String getProto() {
        return proto;
    }

    public void setProto(String proto) {
        this.proto = proto;
    }

    public String getdIP() {
        return dIP;
    }

    public void setdIP(String dIP) {
        this.dIP = dIP;
    }

    public String getsIP() {
        return sIP;
    }

    public void setsIP(String sIP) {
        this.sIP = sIP;
    }

    public String getDport() {
        return dport;
    }

    public void setDport(String dport) {
        this.dport = dport;
    }

    public String getSport() {
        return sport;
    }

    public void setSport(String sport) {
        this.sport = sport;
    }

    public Objeto getDest() {
        return dest;
    }

    public String getDestString() {
        if (dest instanceof Qdisc) {
            Qdisc c = (Qdisc) dest;
            return c.getHandle();
        } else if (dest instanceof Classe) {
            Classe pc = (Classe) dest;
            return pc.getClassid();
        }
        return "";
    }

    public void setDest(Objeto dest) {
        this.dest = dest;
    }

    /**
     * Função que gera o comando TC para um objeto.
     * @return retorna o comando TC.
     */
    public String geraComandosTC() {
        String comando = "iptables -t mangle -A POSTROUTING -o " + scene.getIface();
        String auxMultiport = "";

        comando = comando + " -p " + proto.toLowerCase();

        if ((dIP != null) && (!"".equals(dIP))) {
            comando = comando + " -d " + dIP;
        }
        if ((sIP != null) && (!"".equals(sIP))) {
            comando = comando + " -s " + sIP;
        }
        if ((dport != null) && (!"".equals(dport))) {
            if (isMultiport(dport)) {
                auxMultiport = " -m multiport --dports " + dport;
                //comando = comando + auxMultiport;
            } else {
                comando = comando + " --dport " + dport;
            }
        }
        if ((sport != null) && (!"".equals(sport))) {
            if (isMultiport(sport)) {
                auxMultiport = auxMultiport + " -m --sports " + sport;
                //comando = comando + " -m --sports " + sport;
            } else {
                comando = comando + " --sport " + sport;
            }
        }
        if (auxMultiport != "") {
            comando = comando + " " + auxMultiport;
        }
        if (dest != null) {
            comando = comando + " -j CLASSIFY --set-class " + getDestString();
        }
        return comando;
    }

    /**
     * Verifica de o usuário digitou multiplas portas.
     * @param s portas que usuário digitou.
     * @return true, se é multipor.
     */
    public boolean isMultiport(String s) {
        for (int i = 0; i < s.length(); i++) {
            int ascii = (int) s.charAt(i);
            if ((ascii == 44) || (ascii == 58)) {
                return true;
            }
        }
        return false;
    }
}
