/**
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package frame;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * Função que criar as máscaras que não deixam o usuário digitar qualquer coisa.
 * @author Luiz Gustavo Ender
 */
public class MaskField extends PlainDocument {

    private int tipo;

    public MaskField() {
    }

    public MaskField(int tipo) {
        super();
        this.tipo = tipo;
    }

    @Override
    public void insertString(int offs, String str, AttributeSet a) throws BadLocationException {
        String s = "";
        switch (tipo) {
            case 1:
                // Permite números e o ":"        
                for (int i = 0; i < str.length(); i++) {
                    int ascii = (int) str.charAt(i);
                    if ((ascii >= 48) && (ascii <= 58)) {
                        s = s + Character.toString(str.charAt(i));
                    }
                }
                super.insertString(offs, s, a);
                break;
            case 2:
                // Permite apenas números
                for (int i = 0; i < str.length(); i++) {
                    int ascii = (int) str.charAt(i);
                    if ((ascii >= 48) && (ascii <= 57)) {
                        s = s + Character.toString(str.charAt(i));
                    }
                }
                super.insertString(offs, s, a);
                break;
            case 3:
                // Permite números, "." e "/"
                for (int i = 0; i < str.length(); i++) {
                    int ascii = (int) str.charAt(i);
                    if ((ascii >= 46) && (ascii <= 57)) {
                        s = s + Character.toString(str.charAt(i));
                    }
                }
                super.insertString(offs, s, a);
                break;
            case 4:
                // Permite números, "," e ":"
                for (int i = 0; i < str.length(); i++) {
                    int ascii = (int) str.charAt(i);
                    if ((ascii >= 48) && (ascii <= 58) || (ascii == 44)) {
                        s = s + Character.toString(str.charAt(i));
                    }
                }
                super.insertString(offs, s, a);
        }
    }
}