/**
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package frame;

import elements.Qdisc;
import elements.Objeto;
import elements.Classe;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.*;
import org.netbeans.api.visual.action.PopupMenuProvider;
import org.netbeans.api.visual.widget.Widget;

/**
 * Classe que cria o menu de cada objeto.
 * @author Luiz Gustavo Ender
 */
public class NodeMenu implements PopupMenuProvider, ActionListener {

    private static final String DELETAR = "delete"; // NOI18N
    private static final String CONF = "configurar";
    private static final String COPY = "copy";
    private static final String FILTRO = "filter";
    private JPopupMenu menu;
    private Objeto objecto;
    private Point point;
    private VisualScene scene;
    private JDialog jDprop;

    public NodeMenu(VisualScene scene) {
        this.scene = scene;
        this.jDprop = new JDialog();
        menu = new JPopupMenu("Menu");
        JMenuItem item;
        JPopupMenu.Separator separator;

        item = new JMenuItem("Add Filtro");
        item.setActionCommand(FILTRO);
        item.addActionListener(this);
        menu.add(item);
        separator = new JPopupMenu.Separator();
        this.menu.add(separator);

        item = new JMenuItem("Copiar");
        item.setActionCommand(COPY);
        item.addActionListener(this);
        menu.add(item);
        separator = new JPopupMenu.Separator();
        this.menu.add(separator);

        item = new JMenuItem("Deletar");
        item.setActionCommand(DELETAR);
        item.addActionListener(this);
        menu.add(item);
        separator = new JPopupMenu.Separator();
        this.menu.add(separator);

        item = new JMenuItem("Propriedades");
        item.setActionCommand(CONF);
        item.addActionListener(this);
        menu.add(item);
    }

    @Override
    public JPopupMenu getPopupMenu(Widget widget, Point point) {
        this.point = point;
        this.objecto = (Objeto) widget;
        return menu;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals(DELETAR)) {
            scene.deleteSelectedNodes(false);
        } else if (e.getActionCommand().equals(CONF)) {
            objecto.geraPainelConf();
        } else if (e.getActionCommand().equals(COPY)) {
            scene.copyObjeto();
        } else if (e.getActionCommand().equals(FILTRO)) {
            if (!"qdisc.png".equals(objecto.getImagemNome())) {
                String id = "";
                if (objecto instanceof Qdisc) {
                    id = ((Qdisc) objecto).getHandle();
                } else if (objecto instanceof Classe) {
                    id = ((Classe) objecto).getClassid();
                }
                scene.getFrame().newFilter(id);
            }
        }
    }
}
