/**
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package frame;

import elements.ClassFul;
import elements.Objeto;
import elements.Classe;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import org.netbeans.api.visual.action.PopupMenuProvider;
import org.netbeans.api.visual.widget.Widget;

/**
 * Classe que cria o menu do ambiente de trabalho.
 * 
 * @author Luiz Gustavo Ender
 */
public class SceneMenu implements PopupMenuProvider, ActionListener {

    private static final String ADD_QDISC_ROOT = "addQdiscRoot";
    private static final String ADD_QDISC_SUB = "addQdiscSub";
    private static final String ADD_CLASS_ROOT = "addClassrRoot";
    private static final String ADD_CLASS_SUB = "addClassSub";
    private static final String ADD_NOTE = "AddNote";
    private static final String PASTE = "paste";
    private VisualScene scene;
    private JPopupMenu menu;
    private Point point;
    private int contNode;

    public SceneMenu(VisualScene scene) {
        JMenuItem item;
        JPopupMenu.Separator separator;

        this.scene = scene;
        this.menu = new JPopupMenu("Componentes");
        item = new JMenuItem("Add Qdisc Raiz");
        item.setActionCommand(ADD_QDISC_ROOT);
        item.addActionListener(this);
        this.menu.add(item);
        item = new JMenuItem("Add Qdisc Sub");
        item.setActionCommand(ADD_QDISC_SUB);
        item.addActionListener(this);
        this.menu.add(item);
        separator = new JPopupMenu.Separator();
        this.menu.add(separator);

        item = new JMenuItem("Add Class Raiz");
        item.setActionCommand(ADD_CLASS_ROOT);
        item.addActionListener(this);
        this.menu.add(item);
        item = new JMenuItem("Add Class Sub");
        item.setActionCommand(ADD_CLASS_SUB);
        item.addActionListener(this);
        this.menu.add(item);
        separator = new JPopupMenu.Separator();
        this.menu.add(separator);

        item = new JMenuItem("Add Nota");
        item.setActionCommand(ADD_NOTE);
        item.addActionListener(this);
        this.menu.add(item);
        separator = new JPopupMenu.Separator();
        this.menu.add(separator);

        item = new JMenuItem("Colar");
        item.setActionCommand(PASTE);
        item.addActionListener(this);
        this.menu.add(item);
        this.contNode = 0;
    }

    @Override
    public JPopupMenu getPopupMenu(Widget widget, Point point) {
        this.point = point;
        return this.menu;
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        String imagem = null;

        if (ADD_QDISC_ROOT.equals(e.getActionCommand())) {
            imagem = "qdisc.png";
        } else if (ADD_QDISC_SUB.equals(e.getActionCommand())) {
            imagem = "qdisc_sub.png";
        } else if (ADD_CLASS_ROOT.equals(e.getActionCommand())) {
            imagem = "class.png";
        } else if (ADD_CLASS_SUB.equals(e.getActionCommand())) {
            imagem = "class_sub.png";
        } else if (PASTE.equals(e.getActionCommand())) {
            scene.pasteObjeto(point.x, point.y);
            scene.validate();
            scene.repaint();
            return;
        } else if (ADD_NOTE.equals(e.getActionCommand())) {
            scene.createNote("", point, new Dimension(80, 40));
            scene.validate();
            scene.repaint();
            return;
        }

        ObjetoNode on = new ObjetoNode(imagem);
        Objeto node = (Objeto) scene.addNode(on);
        int x = point.x - (on.getImagem().getIconWidth() / 2);
        int y = point.y - (on.getImagem().getIconHeight() / 2);
        on.setPos(new Point(x, y));
        scene.getSceneAnimator().animatePreferredLocation(node, node.convertLocalToScene(new Point(x, y)));
        scene.setNodeIDcounter(scene.getNodeIDcounter() + 1);
        if (node instanceof ClassFul) {
            if (((ClassFul) node).getQdisc().equals("prio")) {
                for (int i = 1; i <= 3; i++) {
                    ObjetoNode p = new ObjetoNode("prio");
                    p.setPos(on.getPos());
                    Classe prio = new Classe(scene);
                    prio = (Classe) scene.addNode(p);
                    prio.setBand(i);
                    if (i == 1) {
                        x = p.getPos().x - 90;
                    } else if (i == 2) {
                        x = p.getPos().x;
                    } else {
                        x = p.getPos().x + 90;
                    }
                    y = (p.getPos().y + 120);
                    scene.setNodeIDcounter(scene.getNodeIDcounter() + 1);
                    scene.getSceneAnimator().animatePreferredLocation(prio, prio.convertLocalToScene(new Point(x, y)));
                    scene.getSCP().createConnection(node, prio);
                }
            }
        }
        scene.validate();
        scene.repaint();
    }
}
