/**
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob
 * os termos da Licença Pública Geral GNU, conforme publicada pela Free Software
 * Foundation; tanto a versão 2 da Licença como (a seu critério) qualquer versão
 * mais nova.
 *
 * Este programa é distribuído na expectativa de ser útil, mas SEM QUALQUER
 * GARANTIA; sem mesmo a garantia implícita de COMERCIALIZAÇÃO ou de ADEQUAÇÃO A
 * QUALQUER PROPÓSITO EM PARTICULAR. Consulte a Licença Pública Geral GNU para
 * obter mais detalhes.
 *
 * Você deve ter recebido uma cópia da Licença Pública Geral GNU junto com este
 * programa; se não, escreva para a Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package frame;

import elements.Qdisc;
import elements.ClassFul;
import elements.Filter;
import elements.Objeto;
import elements.Classe;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Point;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.dnd.*;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JToggleButton;
import javax.swing.ListSelectionModel;
import javax.swing.ScrollPaneConstants;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.table.DefaultTableModel;

/**
 * Frame principal do VisualTC
 *
 * @author Luiz Gustavo Ender
 */
public class VisualInterface extends javax.swing.JFrame {
    
    private VisualScene scene;
    private JComponent view;
    DataFlavor dataFlavor;
    
    public VisualInterface() {
        initComponents();
        this.setLocationRelativeTo(null);
        setExtendedState(JFrame.MAXIMIZED_BOTH);
        setIconImage((new ImageIcon(getClass().getResource("/imagens/logo_IFSC.png")).getImage()));
        
        dataFlavor = new DataFlavor(ObjetoNode.class, ObjetoNode.class.getSimpleName());
        
        scene = new VisualScene(this);
        scene.initGrids();
        jTBgrid.setSelected(true);
        view = scene.createView();
        jSPtela.setViewportView(view);

        // Seta as mascaras dos capmpos
        jTFsIP.setDocument(new MaskField(3));
        jTFdIP.setDocument(new MaskField(3));
        jTFsport.setDocument(new MaskField(4));
        jTFdport.setDocument(new MaskField(4));


        // Cria o efeito de arrastar e soltar dos objetos da paleta lateral.
        DragSource ds = new DragSource();
        for (Component jl : jPqdiscPaleta.getComponents()) {
            ds.createDefaultDragGestureRecognizer(jl,
                    DnDConstants.ACTION_COPY, new DragGestureListImp());
        }
        for (Component jl : jPclassPaleta.getComponents()) {
            ds.createDefaultDragGestureRecognizer(jl,
                    DnDConstants.ACTION_COPY, new DragGestureListImp());
        }
        
        MyDropTargetListImp myDropTargetListImp = new MyDropTargetListImp(view);
    }
    
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {
        java.awt.GridBagConstraints gridBagConstraints;

        jScrollPane = new javax.swing.JScrollPane();
        jMenuBar2 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        jMenu2 = new javax.swing.JMenu();
        jToolBar1 = new javax.swing.JToolBar();
        jTPprincipal = new javax.swing.JTabbedPane();
        jPdesenho = new javax.swing.JPanel();
        jPqdiscPaleta = new javax.swing.JPanel();
        jLqdisc = new javax.swing.JLabel();
        jLclassFilha = new javax.swing.JLabel();
        jSeparator1 = new javax.swing.JSeparator();
        jPclassPaleta = new javax.swing.JPanel();
        jLqdiscSub = new javax.swing.JLabel();
        jLclass = new javax.swing.JLabel();
        jSPtela = new javax.swing.JScrollPane();
        jTBgrid = new javax.swing.JToggleButton();
        jBzoomOut = new javax.swing.JButton();
        jBzoomIn = new javax.swing.JButton();
        jLstatus = new javax.swing.JLabel();
        jPfilter = new javax.swing.JPanel();
        jPdestino = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        jTFsIP = new javax.swing.JTextField();
        jLabel5 = new javax.swing.JLabel();
        jLhelp4 = new javax.swing.JLabel();
        jTFsport = new javax.swing.JTextField();
        jLheLp6 = new javax.swing.JLabel();
        jPorigem = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jTFdport = new javax.swing.JTextField();
        jLhelp7 = new javax.swing.JLabel();
        jLhelp5 = new javax.swing.JLabel();
        jTFdIP = new javax.swing.JTextField();
        jBaddFilter = new javax.swing.JButton();
        jScrollPane1 = new javax.swing.JScrollPane();
        jTfilters = new javax.swing.JTable();
        jPconf = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jTFiface = new javax.swing.JTextField();
        jCBproto = new javax.swing.JComboBox();
        jCBdest = new javax.swing.JComboBox();
        jLhelp1 = new javax.swing.JLabel();
        jLhelp2 = new javax.swing.JLabel();
        jLhelp3 = new javax.swing.JLabel();
        jBmoveUp = new javax.swing.JButton();
        jBmoveDown = new javax.swing.JButton();
        jBrmFilter = new javax.swing.JButton();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMarquivo = new javax.swing.JMenu();
        jMInew = new javax.swing.JMenuItem();
        jSeparator4 = new javax.swing.JPopupMenu.Separator();
        jMIsave = new javax.swing.JMenuItem();
        jMIsaveAs = new javax.swing.JMenuItem();
        jMIload = new javax.swing.JMenuItem();
        jSeparator3 = new javax.swing.JPopupMenu.Separator();
        jMIsh = new javax.swing.JMenuItem();
        jMIexport = new javax.swing.JMenuItem();
        jSeparator5 = new javax.swing.JPopupMenu.Separator();
        jMenuItem2 = new javax.swing.JMenuItem();
        jMenu3 = new javax.swing.JMenu();
        jMIeditIface = new javax.swing.JMenuItem();
        jSeparator7 = new javax.swing.JPopupMenu.Separator();
        jMIdelete = new javax.swing.JMenuItem();
        jSeparator2 = new javax.swing.JPopupMenu.Separator();
        jMIselecAll = new javax.swing.JMenuItem();
        jMenu4 = new javax.swing.JMenu();
        jMImanual = new javax.swing.JMenuItem();
        jMIsobre = new javax.swing.JMenuItem();

        jScrollPane.setPreferredSize(new java.awt.Dimension(400, 500));

        jMenu1.setText("File");
        jMenuBar2.add(jMenu1);

        jMenu2.setText("Edit");
        jMenuBar2.add(jMenu2);

        jToolBar1.setRollover(true);

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("VisualTC");
        setMinimumSize(new java.awt.Dimension(800, 600));
        setName("VisualTC"); // NOI18N

        jTPprincipal.setEnabled(false);
        jTPprincipal.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                jTPprincipalStateChanged(evt);
            }
        });

        jPqdiscPaleta.setBorder(javax.swing.BorderFactory.createTitledBorder("Qdisc"));
        jPqdiscPaleta.setLayout(new java.awt.GridBagLayout());

        jLqdisc.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/qdisc.png"))); // NOI18N
        jLqdisc.setToolTipText("Qdisc Root");
        jLqdisc.setEnabled(false);
        jLqdisc.setName("qdisc.png"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.ipady = 4;
        gridBagConstraints.insets = new java.awt.Insets(11, 10, 0, 10);
        jPqdiscPaleta.add(jLqdisc, gridBagConstraints);

        jLclassFilha.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/qdisc_sub.png"))); // NOI18N
        jLclassFilha.setToolTipText("Qdisc Filha");
        jLclassFilha.setEnabled(false);
        jLclassFilha.setName("qdisc_sub.png"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 14, 10);
        jPqdiscPaleta.add(jLclassFilha, gridBagConstraints);
        jPqdiscPaleta.add(jSeparator1, new java.awt.GridBagConstraints());

        jPclassPaleta.setBorder(javax.swing.BorderFactory.createTitledBorder("Classe"));
        jPclassPaleta.setLayout(new java.awt.GridBagLayout());

        jLqdiscSub.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/class.png"))); // NOI18N
        jLqdiscSub.setToolTipText("Classe Root");
        jLqdiscSub.setEnabled(false);
        jLqdiscSub.setName("class.png"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(11, 10, 0, 11);
        jPclassPaleta.add(jLqdiscSub, gridBagConstraints);

        jLclass.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/class_sub.png"))); // NOI18N
        jLclass.setToolTipText("Classe Filha");
        jLclass.setEnabled(false);
        jLclass.setName("class_sub.png"); // NOI18N
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.NORTHWEST;
        gridBagConstraints.insets = new java.awt.Insets(5, 10, 15, 11);
        jPclassPaleta.add(jLclass, gridBagConstraints);

        jSPtela.setBackground(Color.WHITE);
        jSPtela.setMinimumSize(new java.awt.Dimension(0, 0));
        jSPtela.setPreferredSize(new java.awt.Dimension(0, 0));
        jSPtela.setRequestFocusEnabled(false);

        jTBgrid.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/dots.png"))); // NOI18N
        jTBgrid.setToolTipText("Habilita/Desabilita Grade");
        jTBgrid.setEnabled(false);
        jTBgrid.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        jTBgrid.setPreferredSize(new java.awt.Dimension(0, 0));
        jTBgrid.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTBgridActionPerformed(evt);
            }
        });

        jBzoomOut.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/zoom_out.png"))); // NOI18N
        jBzoomOut.setToolTipText("Zoom -");
        jBzoomOut.setEnabled(false);
        jBzoomOut.setPreferredSize(new java.awt.Dimension(0, 0));
        jBzoomOut.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBzoomOutActionPerformed(evt);
            }
        });

        jBzoomIn.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/zoom_in.png"))); // NOI18N
        jBzoomIn.setToolTipText("Zoom +");
        jBzoomIn.setEnabled(false);
        jBzoomIn.setPreferredSize(new java.awt.Dimension(0, 0));
        jBzoomIn.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBzoomInActionPerformed(evt);
            }
        });

        jLstatus.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);

        javax.swing.GroupLayout jPdesenhoLayout = new javax.swing.GroupLayout(jPdesenho);
        jPdesenho.setLayout(jPdesenhoLayout);
        jPdesenhoLayout.setHorizontalGroup(
            jPdesenhoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPdesenhoLayout.createSequentialGroup()
                .addGroup(jPdesenhoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPqdiscPaleta, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jPclassPaleta, javax.swing.GroupLayout.PREFERRED_SIZE, 96, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPdesenhoLayout.createSequentialGroup()
                        .addComponent(jTBgrid, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jBzoomIn, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jBzoomOut, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addGap(0, 0, 0)
                .addComponent(jSPtela, javax.swing.GroupLayout.DEFAULT_SIZE, 930, Short.MAX_VALUE))
            .addComponent(jLstatus, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        jPdesenhoLayout.setVerticalGroup(
            jPdesenhoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
            .addGroup(jPdesenhoLayout.createSequentialGroup()
                .addGroup(jPdesenhoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(jPdesenhoLayout.createSequentialGroup()
                        .addGap(25, 25, 25)
                        .addComponent(jPqdiscPaleta, javax.swing.GroupLayout.PREFERRED_SIZE, 200, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jPclassPaleta, javax.swing.GroupLayout.PREFERRED_SIZE, 195, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 233, Short.MAX_VALUE)
                        .addGroup(jPdesenhoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jTBgrid, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jBzoomIn, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(jPdesenhoLayout.createSequentialGroup()
                        .addGap(0, 0, Short.MAX_VALUE)
                        .addComponent(jBzoomOut, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jSPtela, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                .addGap(0, 0, 0)
                .addComponent(jLstatus, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE))
        );

        jPqdiscPaleta.getAccessibleContext().setAccessibleParent(this);

        jTPprincipal.addTab("Hierarquia", jPdesenho);

        jPdestino.setBorder(javax.swing.BorderFactory.createTitledBorder("Origem"));
        jPdestino.setPreferredSize(new java.awt.Dimension(260, 82));

        jLabel4.setText("IP:");

        jLabel5.setText("Porta:");

        jLhelp4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/duvida2.gif"))); // NOI18N
        jLhelp4.setToolTipText("<html>\n      Define o endereço IP origem.<br>\n                &nbsp; - Exemplo: 200.135.37.64/26\"\n</html>");

        jLheLp6.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/duvida2.gif"))); // NOI18N
        jLheLp6.setToolTipText("<html>\n      Define o número da porta de orgem.<br>\n                &nbsp; - Exemplo: 53\n</html>");

        javax.swing.GroupLayout jPdestinoLayout = new javax.swing.GroupLayout(jPdestino);
        jPdestino.setLayout(jPdestinoLayout);
        jPdestinoLayout.setHorizontalGroup(
            jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPdestinoLayout.createSequentialGroup()
                .addGroup(jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel4)
                    .addComponent(jLabel5))
                .addGap(1, 1, 1)
                .addGroup(jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jTFsIP, javax.swing.GroupLayout.DEFAULT_SIZE, 240, Short.MAX_VALUE)
                    .addComponent(jTFsport))
                .addGap(2, 2, 2)
                .addGroup(jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLhelp4)
                    .addComponent(jLheLp6)))
        );
        jPdestinoLayout.setVerticalGroup(
            jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPdestinoLayout.createSequentialGroup()
                .addGap(1, 1, 1)
                .addGroup(jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLheLp6, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addGroup(jPdestinoLayout.createSequentialGroup()
                        .addGroup(jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLhelp4, javax.swing.GroupLayout.PREFERRED_SIZE, 26, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                                .addComponent(jLabel4)
                                .addComponent(jTFsIP, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(1, 1, 1)
                        .addGroup(jPdestinoLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel5)
                            .addComponent(jTFsport, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))))
                .addGap(0, 24, Short.MAX_VALUE))
        );

        jPorigem.setBorder(javax.swing.BorderFactory.createTitledBorder("Destino"));
        jPorigem.setPreferredSize(new java.awt.Dimension(290, 100));

        jLabel6.setText("IP:");

        jLabel7.setText("Porta:");

        jLhelp7.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/duvida2.gif"))); // NOI18N
        jLhelp7.setToolTipText("<html>\n      Define o número da porta de destino.<br>\n                &nbsp; - Exemplo: 53\n</html>");

        jLhelp5.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/duvida2.gif"))); // NOI18N
        jLhelp5.setToolTipText("<html>\n      Define o endereço IP destino.<br>\n                &nbsp; - Exemplo: 8.8.8.8\n</html>");

        javax.swing.GroupLayout jPorigemLayout = new javax.swing.GroupLayout(jPorigem);
        jPorigem.setLayout(jPorigemLayout);
        jPorigemLayout.setHorizontalGroup(
            jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPorigemLayout.createSequentialGroup()
                .addGroup(jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel6)
                    .addComponent(jLabel7))
                .addGap(1, 1, 1)
                .addGroup(jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jTFdport, javax.swing.GroupLayout.DEFAULT_SIZE, 239, Short.MAX_VALUE)
                    .addComponent(jTFdIP))
                .addGap(2, 2, 2)
                .addGroup(jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLhelp7)
                    .addComponent(jLhelp5)))
        );
        jPorigemLayout.setVerticalGroup(
            jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPorigemLayout.createSequentialGroup()
                .addGroup(jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLhelp5, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel6)
                        .addComponent(jTFdIP)))
                .addGap(1, 1, 1)
                .addGroup(jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(jPorigemLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel7)
                        .addComponent(jTFdport, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addComponent(jLhelp7, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(0, 0, Short.MAX_VALUE))
        );

        jLhelp5.getAccessibleContext().setAccessibleDescription("");

        jBaddFilter.setText("Adicionar");
        jBaddFilter.setToolTipText("Adicionar Filtro");
        jBaddFilter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBaddFilterActionPerformed(evt);
            }
        });

        jTfilters.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Protocolo", "IP Origem", "Port Origem", "IP Destino", "Port Destino", "Destino"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class, java.lang.String.class
            };
            boolean[] canEdit = new boolean [] {
                false, false, false, false, false, false
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }

            public boolean isCellEditable(int rowIndex, int columnIndex) {
                return canEdit [columnIndex];
            }
        });
        jTfilters.getTableHeader().setReorderingAllowed(false);
        jTfilters.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseClicked(java.awt.event.MouseEvent evt) {
                jTfiltersMouseClicked(evt);
            }
        });
        jScrollPane1.setViewportView(jTfilters);

        jLabel1.setText("Interface:");

        jLabel2.setText("Protocolo:");

        jLabel3.setText("Destino:");

        jTFiface.setEditable(false);

        jCBproto.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "TCP", "UDP" }));

        jLhelp1.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/duvida2.gif"))); // NOI18N
        jLhelp1.setToolTipText("<html>\n                Mostra a interface que será utilizada.\n</html>");

        jLhelp2.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/duvida2.gif"))); // NOI18N
        jLhelp2.setToolTipText("<html>\n      Define o tipo de protocolo de transporte.<br>\n                &nbsp; - Exemplo: \n</html>");

        jLhelp3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/duvida2.gif"))); // NOI18N
        jLhelp3.setToolTipText("<html>\n      Define para qual Qdisc/Classe o pacote será encaminhado.\n</html>");

        javax.swing.GroupLayout jPconfLayout = new javax.swing.GroupLayout(jPconf);
        jPconf.setLayout(jPconfLayout);
        jPconfLayout.setHorizontalGroup(
            jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPconfLayout.createSequentialGroup()
                .addGap(0, 203, Short.MAX_VALUE)
                .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLhelp1, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLhelp2, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jLhelp3, javax.swing.GroupLayout.Alignment.TRAILING)))
            .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPconfLayout.createSequentialGroup()
                    .addContainerGap()
                    .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                        .addComponent(jLabel1)
                        .addComponent(jLabel2)
                        .addComponent(jLabel3))
                    .addGap(5, 5, 5)
                    .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                        .addComponent(jTFiface)
                        .addComponent(jCBproto, 0, 123, Short.MAX_VALUE)
                        .addComponent(jCBdest, javax.swing.GroupLayout.PREFERRED_SIZE, 123, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );
        jPconfLayout.setVerticalGroup(
            jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPconfLayout.createSequentialGroup()
                .addGap(15, 15, 15)
                .addComponent(jLhelp1)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLhelp2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLhelp3)
                .addContainerGap(35, Short.MAX_VALUE))
            .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                .addGroup(jPconfLayout.createSequentialGroup()
                    .addGap(10, 10, 10)
                    .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel1, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jTFiface, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(1, 1, 1)
                    .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel2, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jCBproto, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addGap(1, 1, 1)
                    .addGroup(jPconfLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                        .addComponent(jLabel3, javax.swing.GroupLayout.PREFERRED_SIZE, 16, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addComponent(jCBdest, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                    .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)))
        );

        jBmoveUp.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/s_cima.png"))); // NOI18N
        jBmoveUp.setToolTipText("Mover p/ cima");
        jBmoveUp.setMaximumSize(new java.awt.Dimension(17, 28));
        jBmoveUp.setMinimumSize(new java.awt.Dimension(17, 28));
        jBmoveUp.setPreferredSize(new java.awt.Dimension(0, 0));
        jBmoveUp.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBmoveUpActionPerformed(evt);
            }
        });

        jBmoveDown.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/s_baixo.png"))); // NOI18N
        jBmoveDown.setToolTipText("Mover p/ baixo");
        jBmoveDown.setMaximumSize(new java.awt.Dimension(19, 28));
        jBmoveDown.setPreferredSize(new java.awt.Dimension(0, 0));
        jBmoveDown.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBmoveDownActionPerformed(evt);
            }
        });

        jBrmFilter.setIcon(new javax.swing.ImageIcon(getClass().getResource("/imagens/lixo.png"))); // NOI18N
        jBrmFilter.setToolTipText("Excluir FIltro");
        jBrmFilter.setPreferredSize(new java.awt.Dimension(0, 0));
        jBrmFilter.setVerifyInputWhenFocusTarget(false);
        jBrmFilter.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jBrmFilterActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPfilterLayout = new javax.swing.GroupLayout(jPfilter);
        jPfilter.setLayout(jPfilterLayout);
        jPfilterLayout.setHorizontalGroup(
            jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, jPfilterLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPfilterLayout.createSequentialGroup()
                        .addGroup(jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING, false)
                            .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPfilterLayout.createSequentialGroup()
                                .addGap(796, 796, 796)
                                .addComponent(jBaddFilter))
                            .addGroup(jPfilterLayout.createSequentialGroup()
                                .addComponent(jPconf, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                                .addComponent(jPdestino, javax.swing.GroupLayout.PREFERRED_SIZE, 306, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addGap(18, 18, 18)
                                .addComponent(jPorigem, javax.swing.GroupLayout.PREFERRED_SIZE, 305, javax.swing.GroupLayout.PREFERRED_SIZE)))
                        .addGap(0, 103, Short.MAX_VALUE)))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jBmoveUp, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jBmoveDown, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jBrmFilter, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap())
        );
        jPfilterLayout.setVerticalGroup(
            jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPfilterLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jPconf, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addGroup(jPfilterLayout.createSequentialGroup()
                        .addGroup(jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jPdestino, javax.swing.GroupLayout.DEFAULT_SIZE, 102, Short.MAX_VALUE)
                            .addComponent(jPorigem, javax.swing.GroupLayout.DEFAULT_SIZE, 102, Short.MAX_VALUE))
                        .addGap(0, 0, Short.MAX_VALUE)))
                .addGap(7, 7, 7)
                .addComponent(jBaddFilter)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPfilterLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPfilterLayout.createSequentialGroup()
                        .addComponent(jBrmFilter, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 202, Short.MAX_VALUE)
                        .addComponent(jBmoveUp, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addComponent(jBmoveDown, javax.swing.GroupLayout.PREFERRED_SIZE, 27, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap(226, Short.MAX_VALUE))
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 527, Short.MAX_VALUE)))
        );

        jTPprincipal.addTab("Filtro", jPfilter);

        jMarquivo.setText("Arquivo");

        jMInew.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_N, java.awt.event.InputEvent.CTRL_MASK));
        jMInew.setText("Novo projeto");
        jMInew.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMInewActionPerformed(evt);
            }
        });
        jMarquivo.add(jMInew);
        jMarquivo.add(jSeparator4);

        jMIsave.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.CTRL_MASK));
        jMIsave.setText("Salvar");
        jMIsave.setEnabled(false);
        jMIsave.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIsaveActionPerformed(evt);
            }
        });
        jMarquivo.add(jMIsave);

        jMIsaveAs.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_S, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMIsaveAs.setText("Salvar como");
        jMIsaveAs.setEnabled(false);
        jMIsaveAs.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIsaveAsActionPerformed(evt);
            }
        });
        jMarquivo.add(jMIsaveAs);

        jMIload.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_O, java.awt.event.InputEvent.CTRL_MASK));
        jMIload.setText("Abrir projeto");
        jMIload.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIloadActionPerformed(evt);
            }
        });
        jMarquivo.add(jMIload);
        jMarquivo.add(jSeparator3);

        jMIsh.setText("Gerar shell script");
        jMIsh.setEnabled(false);
        jMIsh.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIshActionPerformed(evt);
            }
        });
        jMarquivo.add(jMIsh);

        jMIexport.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_E, java.awt.event.InputEvent.SHIFT_MASK | java.awt.event.InputEvent.CTRL_MASK));
        jMIexport.setText("Exportar para PNG");
        jMIexport.setEnabled(false);
        jMIexport.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIexportActionPerformed(evt);
            }
        });
        jMarquivo.add(jMIexport);
        jMarquivo.add(jSeparator5);

        jMenuItem2.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_F4, java.awt.event.InputEvent.ALT_MASK));
        jMenuItem2.setText("Sair");
        jMarquivo.add(jMenuItem2);

        jMenuBar1.add(jMarquivo);

        jMenu3.setText("Editar");

        jMIeditIface.setText("Editar interface");
        jMIeditIface.setEnabled(false);
        jMIeditIface.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIeditIfaceActionPerformed(evt);
            }
        });
        jMenu3.add(jMIeditIface);
        jMenu3.add(jSeparator7);

        jMIdelete.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_DELETE, 0));
        jMIdelete.setText("Deletar");
        jMIdelete.setEnabled(false);
        jMIdelete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIdeleteActionPerformed(evt);
            }
        });
        jMenu3.add(jMIdelete);
        jMenu3.add(jSeparator2);

        jMIselecAll.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_A, java.awt.event.InputEvent.CTRL_MASK));
        jMIselecAll.setText("Selecionar todos");
        jMIselecAll.setEnabled(false);
        jMIselecAll.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIselecAllActionPerformed(evt);
            }
        });
        jMenu3.add(jMIselecAll);

        jMenuBar1.add(jMenu3);

        jMenu4.setText("Ajuda");

        jMImanual.setText("Manual");
        jMImanual.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMImanualActionPerformed(evt);
            }
        });
        jMenu4.add(jMImanual);

        jMIsobre.setText("Sobre");
        jMIsobre.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jMIsobreActionPerformed(evt);
            }
        });
        jMenu4.add(jMIsobre);

        jMenuBar1.add(jMenu4);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addComponent(jTPprincipal)
                .addGap(0, 0, 0))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addGap(0, 0, 0)
                .addComponent(jTPprincipal, javax.swing.GroupLayout.DEFAULT_SIZE, 744, Short.MAX_VALUE))
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jMIexportActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIexportActionPerformed
        scene.exportToPng();
    }//GEN-LAST:event_jMIexportActionPerformed
    
    private void jMIsaveActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIsaveActionPerformed
        scene.save();
    }//GEN-LAST:event_jMIsaveActionPerformed
    
    private void jMIloadActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIloadActionPerformed
        boolean load = scene.load();
        if (load) {
            scene.setActions();
            enableComponentes();
        }
    }//GEN-LAST:event_jMIloadActionPerformed
    
    private void jMIselecAllActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIselecAllActionPerformed
        scene.selectAll();
    }//GEN-LAST:event_jMIselecAllActionPerformed
    
    private void jMIdeleteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIdeleteActionPerformed
        scene.deleteSelectedNodes(false);
    }//GEN-LAST:event_jMIdeleteActionPerformed
    
    private void jMIshActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIshActionPerformed
        scene.geraSh();
    }//GEN-LAST:event_jMIshActionPerformed
    
    private void jMInewActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMInewActionPerformed
        NewProject novo = new NewProject(this, true);
        novo.setLocationRelativeTo(this);
        novo.setVisible(true);
        
        if (novo.isNewProject()) {
            scene = new VisualScene(this);
            scene.initGrids();
            jTBgrid.setSelected(true);
            scene.setActions();
            enableComponentes();
            view = scene.createView();
            jSPtela.setViewportView(view);
            scene.getView().addMouseListener(new MouseAdapter() {
                public void mouseClicked(MouseEvent e) {
                    scene.getView().requestFocusInWindow();
                }
            });
            
            MyDropTargetListImp myDropTargetListImp = new MyDropTargetListImp(view);
            
            scene.setProjeto(novo.getProjeto());
            scene.setIface(novo.getIface());
            scene.setArquivo(new File(novo.getFile()));
            scene.save();
        }
    }//GEN-LAST:event_jMInewActionPerformed
    
    private void jBzoomOutActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBzoomOutActionPerformed
        scene.setZoomFactor(scene.getZoomFactor() / 1.2);
        scene.validate();
    }//GEN-LAST:event_jBzoomOutActionPerformed
    
    private void jBzoomInActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBzoomInActionPerformed
        scene.setZoomFactor(scene.getZoomFactor() * 1.2);
        scene.validate();
    }//GEN-LAST:event_jBzoomInActionPerformed
    
    private void jBaddFilterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBaddFilterActionPerformed
        if (jBaddFilter.getText().equals("Adicionar")) {
            Filter f = new Filter(jCBproto.getSelectedItem().toString(), jTFdport.getText(),
                    jTFdIP.getText(), jTFsport.getText(), jTFsIP.getText(), scene);
            if (jCBdest.getSelectedItem() != null) {
                f.setDest(scene.getParent(jCBdest.getSelectedItem().toString()));
                scene.getFiltros().add(f);
                
                DefaultTableModel model = (DefaultTableModel) jTfilters.getModel();
                model.addRow(new Object[]{jCBproto.getSelectedItem().toString(),
                    jTFsIP.getText(), jTFsport.getText(), jTFdIP.getText(), jTFdport.getText(), jCBdest.getSelectedItem().toString()});
            } else {
                JOptionPane.showMessageDialog(null, "Favor selecionar um destino para este filtro.",
                        "Erro", JOptionPane.ERROR_MESSAGE);
            }
        } else {
            Filter f = scene.getFiltros().get(jTfilters.getSelectedRow());
            f.setProto(jCBproto.getSelectedItem().toString());
            f.setDport(jTFdport.getText());
            f.setdIP(jTFdIP.getText());
            f.setSport(jTFsport.getText());
            f.setsIP(jTFsIP.getText());
            f.setDest(scene.getParent(jCBdest.getSelectedItem().toString()));
            
            DefaultTableModel model = (DefaultTableModel) jTfilters.getModel();
            model.setValueAt(jCBproto.getSelectedItem().toString(), jTfilters.getSelectedRow(), 0);
            model.setValueAt(jTFsIP.getText(), jTfilters.getSelectedRow(), 1);
            model.setValueAt(jTFsport.getText(), jTfilters.getSelectedRow(), 2);
            model.setValueAt(jTFdIP.getText(), jTfilters.getSelectedRow(), 3);
            model.setValueAt(jTFdport.getText(), jTfilters.getSelectedRow(), 4);
            model.setValueAt(jCBdest.getSelectedItem().toString(), jTfilters.getSelectedRow(), 5);
            
            jBaddFilter.setText("Adicionar");
        }
        
        jCBproto.setSelectedIndex(0);
        jTFsIP.setText("");
        jTFsport.setText("");
        jTFdIP.setText("");
        jTFdport.setText("");
        jCBdest.setSelectedIndex(0);
    }//GEN-LAST:event_jBaddFilterActionPerformed
    
    private void jTPprincipalStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_jTPprincipalStateChanged
        if (jTPprincipal.getSelectedIndex() == 1) {
            ArrayList<Objeto> dest = new ArrayList<Objeto>();
            for (ObjetoNode on : scene.getNodes()) {
                Objeto o = (Objeto) scene.findWidget(on);
                if (!"qdisc.png".equals(o.getImagemNome())) {
                    dest.add(o);
                }
            }
            scene.ordenaPorLabel(dest);
            for (Objeto o : dest) {
                if (o instanceof Qdisc) {
                    if (!"".equals(((Qdisc) o).getHandle())) {
                        jCBdest.addItem(((Qdisc) o).getHandle());
                    }
                } else if (o instanceof Classe) {
                    if (!"".equals(((Classe) o).getClassid())) {
                        jCBdest.addItem(((Classe) o).getClassid());
                    }
                }
            }
            jTFiface.setText(scene.getIface());
        } else {
            //jCBproto.setSelectedIndex(0);
            jCBdest.removeAllItems();
            jTFsIP.setText("");
            jTFsport.setText("");
            jTFdIP.setText("");
            jTFdport.setText("");
        }
        
    }//GEN-LAST:event_jTPprincipalStateChanged
    
    private void jBmoveUpActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBmoveUpActionPerformed
        if ((jTfilters.getSelectedRow() - 1) >= 0) {
            DefaultTableModel model = (DefaultTableModel) jTfilters.getModel();
            model.moveRow(jTfilters.getSelectedRow(), jTfilters.getSelectedRow(), jTfilters.getSelectedRow() - 1);
            
            Collections.rotate(scene.getFiltros().subList(jTfilters.getSelectedRow() - 1, jTfilters.getSelectedRow() + 1), 1);
            
            ListSelectionModel selectionModel = jTfilters.getSelectionModel();
            selectionModel.setSelectionInterval(jTfilters.getSelectedRow() - 1, jTfilters.getSelectedRow() - 1);
        }
    }//GEN-LAST:event_jBmoveUpActionPerformed
    
    private void jBmoveDownActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBmoveDownActionPerformed
        if ((jTfilters.getSelectedRow() + 1) < jTfilters.getRowCount()) {
            DefaultTableModel model = (DefaultTableModel) jTfilters.getModel();
            model.moveRow(jTfilters.getSelectedRow(), jTfilters.getSelectedRow(), jTfilters.getSelectedRow() + 1);
            
            Collections.rotate(scene.getFiltros().subList(jTfilters.getSelectedRow(), jTfilters.getSelectedRow() + 2), 1);
            
            ListSelectionModel selectionModel = jTfilters.getSelectionModel();
            selectionModel.setSelectionInterval(jTfilters.getSelectedRow() + 1, jTfilters.getSelectedRow() + 1);
        }
    }//GEN-LAST:event_jBmoveDownActionPerformed
    
    private void jBrmFilterActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jBrmFilterActionPerformed
        DefaultTableModel model = (DefaultTableModel) jTfilters.getModel();
        
        scene.getFiltros().remove(jTfilters.getSelectedRow());
        model.removeRow(jTfilters.getSelectedRow());
    }//GEN-LAST:event_jBrmFilterActionPerformed
    
    private void jTBgridActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTBgridActionPerformed
        if (jTBgrid.isSelected()) {
            scene.initGrids();
        } else {
            scene.setBackground(Color.WHITE);
            scene.validate();
        }
    }//GEN-LAST:event_jTBgridActionPerformed
    
    private void jTfiltersMouseClicked(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_jTfiltersMouseClicked
        if (evt.getClickCount() == 2) {
            jBaddFilter.setText("Alterar");
            
            Filter f = scene.getFiltros().get(jTfilters.getSelectedRow());
            
            jCBproto.setSelectedItem(f.getProto());
            jTFsIP.setText(f.getsIP());
            jTFsport.setText(f.getSport());
            jTFdIP.setText(f.getdIP());
            jTFdport.setText(f.getDport());
            jCBdest.setSelectedItem(f.getDestString());
        }
    }//GEN-LAST:event_jTfiltersMouseClicked
    
    private void jMIsaveAsActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIsaveAsActionPerformed
        scene.saveAs();
    }//GEN-LAST:event_jMIsaveAsActionPerformed
    
    private void jMIeditIfaceActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIeditIfaceActionPerformed
        String iface = JOptionPane.showInputDialog(this, "A interface atual é: " + scene.getIface()
                + ".\nInforme a nova interface que deseja utilizar.", "Alterar interface",
                JOptionPane.INFORMATION_MESSAGE);
        if (iface != null) {
            while (iface.length() <= 0) {
                JOptionPane.showMessageDialog(this, "Não é permitido valores em branco",
                        "Erro", JOptionPane.ERROR_MESSAGE);
                iface = JOptionPane.showInputDialog(this, "A interface atual é: " + scene.getIface()
                        + ".\nInforme a nova interface que deseja utilizar.", "Alterar interface",
                        JOptionPane.INFORMATION_MESSAGE);
            }
            if (!"".equals(iface)) {
                scene.setIface(iface);
            }
        }
    }//GEN-LAST:event_jMIeditIfaceActionPerformed
    
    private void jMImanualActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMImanualActionPerformed
        final JEditorPane jEPhelp = new JEditorPane();
        jEPhelp.setEditable(false);

        // Abringo pagina principal do manual
        try {
            jEPhelp.setPage(getClass().getResource("/help/index.htm"));
        } catch (IOException e) {
            
        }

        // Evento para abrir links internos do manual
        jEPhelp.addHyperlinkListener(new HyperlinkListener() {
            @Override
            public void hyperlinkUpdate(HyperlinkEvent e) {
                if (HyperlinkEvent.EventType.ACTIVATED.equals(e.getEventType())) {
                    try {
                        jEPhelp.setPage(e.getURL().toString());
                    } catch (IOException ex) {
                        
                    }
                }
            }
        });
        
        JScrollPane jSPhelp = new JScrollPane(jEPhelp,
                ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
                ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        jSPhelp.setPreferredSize(new Dimension(800, 700));
        
        
        JDialog help = new JDialog(this, true);
        help.setBounds(0, 0, 800, 700);
        help.setTitle("Manual - VisualTC");
        help.getContentPane().add(jSPhelp, BorderLayout.CENTER);
        help.setLocationRelativeTo(this);
        help.setVisible(true);
    }//GEN-LAST:event_jMImanualActionPerformed
    
    private void jMIsobreActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jMIsobreActionPerformed
        About about = new About(this, true);
        about.setLocationRelativeTo(this);
        about.setVisible(true);
    }//GEN-LAST:event_jMIsobreActionPerformed
    
    public void clearFiltros() {
        DefaultTableModel model = (DefaultTableModel) jTfilters.getModel();
        
        int i = 0;
        while (i < model.getRowCount()) {
            model.removeRow(i);
            i++;
        }
    }

    /**
     * Função para adicionar um filtro quando for carregado um projeto.
     *
     * @param proto protocolo
     * @param dport porta de destino
     * @param dIP ip destino
     * @param sport porta de origem
     * @param sIP ip de origem
     * @param dest destino dos pacote deste filtro
     */
    public void addFilter(String proto, String dport, String dIP, String sport, String sIP, String dest) {
        Filter f = new Filter(proto, dport, dIP, sport, sIP, scene);
        f.setDest(scene.getParent(dest));
        scene.getFiltros().add(f);
        
        DefaultTableModel model = (DefaultTableModel) jTfilters.getModel();
        model.addRow(new Object[]{proto, sIP, sport, dIP, dport, dest});
    }

    /**
     * Função responsável por setar o destino do filtro. Usado quando o usário
     * adiciona um filtro a partir do meu do objeto.
     *
     * @param id destino
     */
    public void newFilter(String id) {
        jTPprincipal.setSelectedIndex(1);
        jCBdest.setSelectedItem(id);
    }
    
    public void enableComponentes() {
        jMarquivo.setEnabled(true);
        jMIsave.setEnabled(true);
        jMIsaveAs.setEnabled(true);
        jMIsh.setEnabled(true);
        jMIexport.setEnabled(true);
        jMenu3.setEnabled(true);
        jMIeditIface.setEnabled(true);
        jMIdelete.setEnabled(true);
        jMIselecAll.setEnabled(true);
        jTPprincipal.setEnabled(true);
        jLqdisc.setEnabled(true);
        jLclassFilha.setEnabled(true);
        jLqdiscSub.setEnabled(true);
        jLclass.setEnabled(true);
        jTBgrid.setEnabled(true);
        jBzoomIn.setEnabled(true);
        jBzoomOut.setEnabled(true);
    }
    
    public JScrollPane getjSPtela() {
        return jSPtela;
    }

    public JToggleButton getjTBgrid() {
        return jTBgrid;
    }
    
    public VisualScene getScene() {
        return scene;
    }
    
    public JLabel getjLstatus() {
        return jLstatus;
    }
    
    public static void main(String args[]) {
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /*
         * If Nimbus (introduced in Java SE 6) is not available, stay with the
         * default look and feel. For details see
         * http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(VisualInterface.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(VisualInterface.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(VisualInterface.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(VisualInterface.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>
        java.awt.EventQueue.invokeLater(new Runnable() {
            @Override
            public void run() {
                new VisualInterface().setVisible(true);
            }
        });
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jBaddFilter;
    private javax.swing.JButton jBmoveDown;
    private javax.swing.JButton jBmoveUp;
    private javax.swing.JButton jBrmFilter;
    private javax.swing.JButton jBzoomIn;
    private javax.swing.JButton jBzoomOut;
    private javax.swing.JComboBox jCBdest;
    private javax.swing.JComboBox jCBproto;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLclass;
    private javax.swing.JLabel jLclassFilha;
    private javax.swing.JLabel jLheLp6;
    private javax.swing.JLabel jLhelp1;
    private javax.swing.JLabel jLhelp2;
    private javax.swing.JLabel jLhelp3;
    private javax.swing.JLabel jLhelp4;
    private javax.swing.JLabel jLhelp5;
    private javax.swing.JLabel jLhelp7;
    private javax.swing.JLabel jLqdisc;
    private javax.swing.JLabel jLqdiscSub;
    private javax.swing.JLabel jLstatus;
    private javax.swing.JMenuItem jMIdelete;
    private javax.swing.JMenuItem jMIeditIface;
    private javax.swing.JMenuItem jMIexport;
    private javax.swing.JMenuItem jMIload;
    private javax.swing.JMenuItem jMImanual;
    private javax.swing.JMenuItem jMInew;
    private javax.swing.JMenuItem jMIsave;
    private javax.swing.JMenuItem jMIsaveAs;
    private javax.swing.JMenuItem jMIselecAll;
    private javax.swing.JMenuItem jMIsh;
    private javax.swing.JMenuItem jMIsobre;
    private javax.swing.JMenu jMarquivo;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenu jMenu3;
    private javax.swing.JMenu jMenu4;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JMenuBar jMenuBar2;
    private javax.swing.JMenuItem jMenuItem2;
    private javax.swing.JPanel jPclassPaleta;
    private javax.swing.JPanel jPconf;
    private javax.swing.JPanel jPdesenho;
    private javax.swing.JPanel jPdestino;
    private javax.swing.JPanel jPfilter;
    private javax.swing.JPanel jPorigem;
    private javax.swing.JPanel jPqdiscPaleta;
    private javax.swing.JScrollPane jSPtela;
    private javax.swing.JScrollPane jScrollPane;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JPopupMenu.Separator jSeparator2;
    private javax.swing.JPopupMenu.Separator jSeparator3;
    private javax.swing.JPopupMenu.Separator jSeparator4;
    private javax.swing.JPopupMenu.Separator jSeparator5;
    private javax.swing.JPopupMenu.Separator jSeparator7;
    private javax.swing.JToggleButton jTBgrid;
    private javax.swing.JTextField jTFdIP;
    private javax.swing.JTextField jTFdport;
    private javax.swing.JTextField jTFiface;
    private javax.swing.JTextField jTFsIP;
    private javax.swing.JTextField jTFsport;
    private javax.swing.JTabbedPane jTPprincipal;
    private javax.swing.JTable jTfilters;
    private javax.swing.JToolBar jToolBar1;
    // End of variables declaration//GEN-END:variables

    /**
     * Classe responsável pela transferencia de objetos
     */
    class TransferableObjecto implements Transferable {
        
        private ObjetoNode o;
        
        public TransferableObjecto(ObjetoNode ob) {
            this.o = ob;
        }
        
        @Override
        public DataFlavor[] getTransferDataFlavors() {
            return new DataFlavor[]{dataFlavor};
        }
        
        @Override
        public boolean isDataFlavorSupported(DataFlavor flavor) {
            return flavor.equals(dataFlavor);
        }
        
        @Override
        public Object getTransferData(DataFlavor flavor)
                throws UnsupportedFlavorException, IOException {
            
            if (flavor.equals(dataFlavor)) {
                return o;
            } else {
                throw new UnsupportedFlavorException(flavor);
            }
        }
        
        public ObjetoNode getO() {
            return o;
        }
    }

    /**
     * Classe que criar o efeito de arrastar e soltar
     */
    class DragGestureListImp implements DragGestureListener {
        
        @Override
        public void dragGestureRecognized(DragGestureEvent event) {
            Cursor cursor = null;
            JLabel jLob = (JLabel) event.getComponent();
            
            if (event.getDragAction() == DnDConstants.ACTION_COPY) {
                cursor = DragSource.DefaultCopyDrop;
            }
            ObjetoNode ob = new ObjetoNode(jLob.getIcon());
            ob.setNomImagem(jLob.getName());
            event.startDrag(cursor, new TransferableObjecto(ob));
        }
    }

    /**
     * Classe que adiciona um objeto com o usuário soltar o mouse.
     */
    class MyDropTargetListImp extends DropTargetAdapter implements DropTargetListener {
        
        private DropTarget dropTarget;
        private JComponent panel;
        
        public MyDropTargetListImp(JComponent panel) {
            this.panel = panel;
            dropTarget = new DropTarget(this.panel, DnDConstants.ACTION_COPY, this,
                    true, null);
        }
        
        @Override
        public void drop(DropTargetDropEvent event) {
            try {
                Transferable tr = event.getTransferable();
                if (event.isDataFlavorSupported(dataFlavor)) {
                    event.acceptDrop(DnDConstants.ACTION_COPY);
                    ObjetoNode o = (ObjetoNode) tr.getTransferData(dataFlavor);
                    int x = event.getLocation().x - (o.getImagem().getIconWidth() / 2);
                    int y = event.getLocation().y - (o.getImagem().getIconHeight() / 2);
                    scene.setNovo(true);
                    o.setPos(new Point(x, y));
                    Objeto node = (Objeto) scene.addNode(o);
                    if (node == null) {
                        scene.removeNode(o);
                    } else {
                        node.setPreferredLocation(node.convertLocalToScene(new Point(x, y)));
                        scene.setNodeIDcounter(scene.getNodeIDcounter() + 1);
                        scene.getSceneAnimator().animatePreferredLocation(node, new Point(x, y));
                        if (node instanceof ClassFul) {
                            if (((ClassFul) node).getQdisc().equals("prio")) {
                                for (int i = 1; i <= 3; i++) {
                                    ObjetoNode on = new ObjetoNode("prio");
                                    on.setPos(o.getPos());
                                    Classe prio = new Classe(scene);
                                    prio = (Classe) scene.addNode(on);
                                    prio.setBand(i);
                                    if (i == 1) {
                                        x = on.getPos().x - 90;
                                    } else if (i == 2) {
                                        x = on.getPos().x;
                                    } else {
                                        x = on.getPos().x + 90;
                                    }
                                    y = (on.getPos().y + 120);
                                    prio.setPreferredLocation(prio.convertLocalToScene(new Point(x, y)));
                                    scene.setNodeIDcounter(scene.getNodeIDcounter() + 1);
                                    scene.getSceneAnimator().animatePreferredLocation(prio, new Point(x, y));
                                    scene.getSCP().createConnection(node, prio);
                                }
                            }
                        }
                        scene.validate();
                        scene.repaint();
                        this.panel.validate();
                        event.dropComplete(true);
                        return;
                    }
                }
                event.rejectDrop();
            } catch (Exception e) {
            }
        }
    }
}
