/**
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *  Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 *  Este programa é software livre; você pode redistribuí-lo e/ou
 *  modificá-lo sob os termos da Licença Pública Geral GNU, conforme
 *  publicada pela Free Software Foundation; tanto a versão 2 da
 *  Licença como (a seu critério) qualquer versão mais nova.
 *
 *  Este programa é distribuído na expectativa de ser útil, mas SEM
 *  QUALQUER GARANTIA; sem mesmo a garantia implícita de
 *  COMERCIALIZAÇÃO ou de ADEQUAÇÃO A QUALQUER PROPÓSITO EM
 *  PARTICULAR. Consulte a Licença Pública Geral GNU para obter mais
 *  detalhes.
 *
 *  Você deve ter recebido uma cópia da Licença Pública Geral GNU
 *  junto com este programa; se não, escreva para a Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307, USA.
*/

package frame;

import java.awt.Point;
import java.util.HashMap;
import java.util.Map;
import org.netbeans.api.visual.action.MoveProvider;
import org.netbeans.api.visual.widget.Widget;

/**
 * Função que faz mover mais de um objeto ao mesmo tempo.
 * @author Luiz Gustavo Ender
 */
public class VisualMoveProvider implements MoveProvider {

    private HashMap<Widget, Point> originals = new HashMap<Widget, Point>();
    private Point original;
    private VisualScene scene;

    public VisualMoveProvider(VisualScene scene) {
        this.scene = scene;
    }

    @Override
    public void movementStarted(Widget widget) {
        Object object = scene.findObject(widget);
        if (scene.isNode(object)) {
            for (Object o : scene.getSelectedObjects()) {
                if (scene.isNode(o)) {
                    Widget w = scene.findWidget(o);
                    if (w != null) {
                        originals.put(w, w.getPreferredLocation());
                    }
                }
            }
        } else {
            originals.put(widget, widget.getPreferredLocation());
        }
    }

    @Override
    public void movementFinished(Widget widget) {
        originals.clear();
        original = null;
    }

    @Override
    public Point getOriginalLocation(Widget widget) {
        original = widget.getPreferredLocation();
        return original;
    }

    @Override
    public void setNewLocation(Widget widget, Point location) {
        int dx = location.x - original.x;
        int dy = location.y - original.y;
        for (Map.Entry<Widget, Point> entry : originals.entrySet()) {
            Point point = entry.getValue();
            entry.getKey().setPreferredLocation(new Point(point.x + dx, point.y + dy));
        }
    }
}
