/**
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * Copyright (C) 2013 Emerson Ribeiro de Mello e Luiz Gustavo Ender
 *
 * Este programa é software livre; você pode redistribuí-lo e/ou modificá-lo sob
 * os termos da Licença Pública Geral GNU, conforme publicada pela Free Software
 * Foundation; tanto a versão 2 da Licença como (a seu critério) qualquer versão
 * mais nova.
 *
 * Este programa é distribuído na expectativa de ser útil, mas SEM QUALQUER
 * GARANTIA; sem mesmo a garantia implícita de COMERCIALIZAÇÃO ou de ADEQUAÇÃO A
 * QUALQUER PROPÓSITO EM PARTICULAR. Consulte a Licença Pública Geral GNU para
 * obter mais detalhes.
 *
 * Você deve ter recebido uma cópia da Licença Pública Geral GNU junto com este
 * programa; se não, escreva para a Free Software Foundation, Inc., 59 Temple
 * Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package frame;

import elements.Qdisc;
import elements.ClassFul;
import elements.Filter;
import elements.Objeto;
import elements.ClassLess;
import elements.Classe;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.TexturePaint;
import java.awt.Toolkit;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.image.BufferedImage;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.Date;
import java.util.HashSet;
import java.util.Random;
import java.util.Set;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.filechooser.FileNameExtensionFilter;
import org.netbeans.api.visual.action.ActionFactory;
import org.netbeans.api.visual.action.WidgetAction;
import org.netbeans.api.visual.anchor.AnchorFactory;
import org.netbeans.api.visual.anchor.AnchorShape;
import org.netbeans.api.visual.anchor.PointShape;
import org.netbeans.api.visual.graph.GraphScene;
import org.netbeans.api.visual.router.Router;
import org.netbeans.api.visual.router.RouterFactory;
import org.netbeans.api.visual.widget.ComponentWidget;
import org.netbeans.api.visual.widget.ConnectionWidget;
import org.netbeans.api.visual.widget.LayerWidget;
import org.netbeans.api.visual.widget.Widget;
import org.openide.ErrorManager;
import org.openide.util.ImageUtilities;

/**
 * A principal classe do VisualTC, nela é feita todo o gerenciamento do ambiente
 * de trabalho.
 *
 * @author Luiz Gustavo Ender
 */
public class VisualScene extends GraphScene<ObjetoNode, String> {

    private final static String QDISC_PATH = "qdisc.png";
    private final static String QDISC_SUB_PATH = "qdisc_sub.png";
    private LayerWidget mainLayer, connectionLayer, interractionLayer, backgroundLayer;
    private WidgetAction moveAction, connectAction, reconnectAction, moveControlPointAction;
    private WidgetAction moveAlign, resizeAction;
    private VisualConnectProvider SCP;
    private VisualReconnectProvider SRP;
    private Router router;
    private NodeMenu nodeMenu;
    private EdgeMenu edgeMenu;
    private int nodeIDcounter, edgeIDcounter;
    private boolean novo;
    private VisualInterface frame;
    private String projeto, iface, lastLocate, lastFile;
    private File arquivo;
    private ArrayList<Filter> filtros;
    private ArrayList<ComponentWidget> notas;
    private Objeto m_O;
    private ObjetoNode m_On;

    public VisualScene(final VisualInterface frame) {
        this.mainLayer = new LayerWidget(this);
        this.connectionLayer = new LayerWidget(this);
        this.interractionLayer = new LayerWidget(this);
        this.backgroundLayer = new LayerWidget(this);
        this.SCP = new VisualConnectProvider(this);
        this.SRP = new VisualReconnectProvider(this);
        this.connectAction = ActionFactory.createExtendedConnectAction(interractionLayer, SCP);
        this.reconnectAction = ActionFactory.createReconnectAction(SRP);
        this.moveControlPointAction = ActionFactory.createFreeMoveControlPointAction();
        this.moveAlign = ActionFactory.createAlignWithMoveAction(
                mainLayer, interractionLayer, ActionFactory.createDefaultAlignWithMoveDecorator());
        this.resizeAction = ActionFactory.createAlignWithResizeAction(mainLayer, interractionLayer, null);
        this.router = RouterFactory.createFreeRouter();
        this.moveAction = ActionFactory.createMoveAction(null, new VisualMoveProvider(this));
        this.nodeMenu = new NodeMenu(this);
        this.edgeMenu = new EdgeMenu(this);
        this.novo = true;
        this.frame = frame;
        this.projeto = "'vazio'";
        this.filtros = new ArrayList<Filter>();
        this.notas = new ArrayList<ComponentWidget>();

        addChild(this.backgroundLayer);
        addChild(this.mainLayer);
        addChild(this.connectionLayer);
        addChild(this.interractionLayer);

        //Quando clicar com o mouse no ambiente de trabalho, solicitar 
        //focus para o ambiente de trabalho e limpar barra de status.
        getActions().addAction(new WidgetAction.Adapter() {
            @Override
            public WidgetAction.State mouseClicked(Widget wid, WidgetAction.WidgetMouseEvent evt) {
                if (wid instanceof VisualScene) {
                    getView().requestFocusInWindow();
                    frame.getjLstatus().setText("");
                }
                return WidgetAction.State.CONSUMED;
            }
        });
    }

    /**
     * Responsável por criar o objeto no ambiente de trabalho.
     *
     * @param o objeto que contém a imagem
     * @return Caso haja sucesso, retorna o objeto criado. Se não, retorna NULL
     */
    @Override
    protected Widget attachNodeWidget(ObjetoNode o) {
        Objeto objeto = null;
        Image imagem = ImageUtilities.loadImage("/imagens/" + o.getNomImagem());
        String text = "";
        if (novo) {
            if (!"prio".equals(o.getNomImagem())) {
                if (o.getNomImagem().equals(QDISC_PATH) || o.getNomImagem().equals(QDISC_SUB_PATH)) {
                    Object[] opcoes = {"pfifo", "sfq", "tbf", "prio", "htb"};
                    Object res = JOptionPane.showInputDialog(frame, "Selecione uma Qdisc: ", "Informe a Qdisc",
                            JOptionPane.PLAIN_MESSAGE, null, opcoes, "");
                    if (res != null) {
                        String qdisc = res.toString();
                        if (("PRIO".equalsIgnoreCase(qdisc)) || ("HTB".equalsIgnoreCase(qdisc))) {
                            objeto = new ClassFul(this);
                            ((ClassFul) objeto).setQdisc(qdisc);
                        } else {
                            objeto = new ClassLess(this);
                            ((ClassLess) objeto).setQdisc(qdisc);
                        }
                        if (o.getNomImagem().equals(QDISC_PATH)) {
                            String label = defineLabel(1, "", objeto);
                            ((Qdisc) objeto).setHandle(label);
                            objeto.setLabel(label);
                            objeto.setNivel(1);
                        }
                        objeto.setToolTipText("Qdisc " + qdisc.toUpperCase());
                        text = "Qdisc\n" + qdisc.toUpperCase();
                    } else {
                        return null;
                    }
                } else {
                    boolean addclass = false;
                    for (ObjetoNode on : getNodes()) {
                        Objeto c = (Objeto) findWidget(on);
                        if (c instanceof ClassFul) {
                            ClassFul cf = (ClassFul) c;
                            if (cf.getQdisc().equalsIgnoreCase("HTB")) {
                                addclass = true;
                            }
                        }
                    }
                    if (addclass) {
                        objeto = new Classe(this);
                        objeto.setToolTipText("Classe HTB");
                        text = "Classe\nHTB";
                    } else {
                        JOptionPane.showMessageDialog(frame, "Não possível adicionar uma Classe sem ter uma Qdisc ClassFul do tipo HTB criada.",
                                "Erro", JOptionPane.ERROR_MESSAGE);
                        return null;
                    }
                }
            } else {
                objeto = new Classe(this);
                ((Classe) objeto).setNomClass("prio");
                objeto.setToolTipText("Classe PRIO");
                text = "Classe\nPRIO";
                imagem = ImageUtilities.loadImage("/imagens/class.png");
                o.setNomImagem("class.png");
            }
        } else {
            if (o.getTipo().equals("ClassFul")) {
                objeto = new ClassFul(this);
            } else if (o.getTipo().equals("ClassLess")) {
                objeto = new ClassLess(this);
            } else if (o.getTipo().equals("Classe")) {
                objeto = new Classe(this);
            }
        }

        objeto.getActions().addAction(new WidgetAction.Adapter() {
            @Override
            public State mouseClicked(Widget wid, WidgetMouseEvent evt) {
                if (wid instanceof Objeto) {
                    frame.getjLstatus().setText("Para criar um conexão efetue: Ctrl + Clique e arraste para o destino.");
                }
                return State.CONSUMED;
            }
        });
        objeto.setImage(insertTextImage(imagem, text));
        objeto.setImagemNome(o.getNomImagem());
        objeto.getActions().addAction(createSelectAction());
        objeto.getActions().addAction(createObjectHoverAction());
        objeto.getActions().addAction(connectAction);
        objeto.getActions().addAction(moveAlign);
        //objeto.getActions().addAction(moveAction); //adiciona a ação que permite mover vários objetos junto
        objeto.getActions().addAction(ActionFactory.createPopupMenuAction(nodeMenu));

        mainLayer.addChild(objeto);
        return objeto;
    }

    @Override
    protected Widget attachEdgeWidget(String e) {
        ConnectionWidget conecta = new ConnectionWidget(this);
        conecta.setRouter(router);
        conecta.setTargetAnchorShape(AnchorShape.TRIANGLE_FILLED);
        conecta.setControlPointShape(PointShape.SQUARE_FILLED_BIG);
        conecta.setEndPointShape(PointShape.SQUARE_FILLED_BIG);
        connectionLayer.addChild(conecta);
        conecta.getActions().addAction(reconnectAction);
        conecta.getActions().addAction(createSelectAction());
        conecta.getActions().addAction(ActionFactory.createAddRemoveControlPointAction());
        conecta.getActions().addAction(moveControlPointAction);
        conecta.getActions().addAction(ActionFactory.createPopupMenuAction(edgeMenu));
        return conecta;
    }

    @Override
    protected void attachEdgeSourceAnchor(String e, ObjetoNode n, ObjetoNode n1) {
        ConnectionWidget widget = (ConnectionWidget) findWidget(e);
        Widget sourceNodeWidget = findWidget(n1);
        widget.setSourceAnchor(sourceNodeWidget != null ? AnchorFactory.createFreeRectangularAnchor(sourceNodeWidget, true) : null);
    }

    @Override
    protected void attachEdgeTargetAnchor(String e, ObjetoNode n, ObjetoNode n1) {
        ConnectionWidget widget = (ConnectionWidget) findWidget(e);
        Widget targetNodeWidget = findWidget(n1);
        widget.setTargetAnchor(targetNodeWidget != null ? AnchorFactory.createFreeRectangularAnchor(targetNodeWidget, true) : null);
    }

    /**
     * Criar a "grade" no ambiente de trabalho.
     */
    public void initGrids() {
        Image sourceImage = ImageUtilities.loadImage("imagens/grid.png"); // NOI18N
        int width = sourceImage.getWidth(null);
        int height = sourceImage.getHeight(null);
        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        Graphics2D graphics = image.createGraphics();
        graphics.drawImage(sourceImage, 0, 0, null);
        graphics.dispose();
        TexturePaint PAINT_BACKGROUND = new TexturePaint(image, new Rectangle(0, 0, width, height));
        setBackground(PAINT_BACKGROUND);
        repaint();
        revalidate(false);
        validate();
    }

    /**
     * Insere um texto em uma imagem.
     *
     * @param img imagem que será inserido o texto
     * @param text texto que será inserido na imagem
     * @return retorna a imgem com o texto
     */
    public Image insertTextImage(Image img, String text) {
        BufferedImage bi = new BufferedImage(img.getWidth(null), img.getHeight(null), BufferedImage.TYPE_INT_ARGB);
        Graphics2D g = (Graphics2D) bi.getGraphics();
        g.drawImage(img, 0, 0, null);
        g.setFont(getFont());
        g.setColor(getForeground());
        g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_DEFAULT);
        g.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING, RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
        int x = img.getWidth(null) / 2;
        int y = 20;
        for (String line : text.split("\n")) {
            int stringLen = (int) g.getFontMetrics().getStringBounds(line, g).getWidth();
            int start = x - (stringLen / 2);
            g.drawString(line, start, y += g.getFontMetrics().getHeight());
        }
        return Toolkit.getDefaultToolkit().createImage(bi.getSource());
    }

    /**
     * Responsável por criar as notas no ambiente de trabalho.
     *
     * @param label texto inserido na nota
     * @param location localização onde será criado a nota na tela
     * @param d dimensão da nota.
     */
    public void createNote(String label, Point location, Dimension d) {
        final JTextArea jText = new JTextArea(label);
        jText.setOpaque(true);
        jText.setLineWrap(true);
        jText.setBackground(Color.YELLOW);

        final ComponentWidget widget = new ComponentWidget(this, jText);
        widget.setCursor(new Cursor(Cursor.MOVE_CURSOR));
        widget.setPreferredSize(d);
        widget.setOpaque(true);
        widget.setBackground(Color.WHITE);
        widget.setBorder(org.netbeans.api.visual.border.BorderFactory.createEmptyBorder(8));
        widget.setPreferredLocation(location);
        widget.getActions().addAction(ActionFactory.createPopupMenuAction(new NoteMenu(this)));
        widget.getActions().addAction(createSelectAction());
        widget.getActions().addAction(resizeAction);
        widget.getActions().addAction(moveAction);

        jText.addFocusListener(new FocusAdapter() {
            public void focusLost(FocusEvent evt) {
                jText.setSelectionStart(0);
                jText.setSelectionEnd(0);
                widget.setBorder(org.netbeans.api.visual.border.BorderFactory.createEmptyBorder(8));
                validate();
                repaint();
            }
        });
        jText.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                widget.setBorder(org.netbeans.api.visual.border.BorderFactory.createResizeBorder(8));
                validate();
                repaint();
            }
        });

        notas.add(widget);

        mainLayer.addChild(widget);
    }

    /**
     * Está função deleta os objetos e conexões selecionados assim, caso o
     * usuário deseje, os objetos que estão abaixo dele no nível de hierarquia.
     *
     * @param reconnect true, remover a conexão atinga. False, não remove.
     */
    public void deleteSelectedNodes(boolean reconnect) {
        Set<ObjetoNode> delNode = new HashSet<ObjetoNode>();
        Set<Objeto> delCascata = new HashSet<Objeto>();
        Set<String> delEdge = new HashSet<String>();
        String niveis = "";

        // Adiciona o obejeto seleciona em seu devido arraylisy
        for (Object o : getSelectedObjects()) {
            if (isNode(o)) {
                delNode.add((ObjetoNode) o);
            } else if (isEdge(o)) {
                delEdge.add(o.toString());
            }
        }

        // Se não selecinou todos os objetos
        // Verifica se o usuário deseja deletar também os objetos abaixo do objeto selecionado
        if (delNode.size() < getNodes().size()) {
            for (ObjetoNode on : delNode) {
                Objeto o = (Objeto) findWidget(on);
                Set<Objeto> cascata = new HashSet<Objeto>();
                cascata.addAll(getChildrenObjeto(o));
                if (o instanceof ClassFul) {
                    if (((ClassFul) o).getQdisc().equals("prio")) {
                        Set<Objeto> prio = new HashSet<Objeto>();
                        for (Objeto objeto : cascata) {
                            if (objeto instanceof Classe) {
                                if (((Classe) objeto).getNomClass().equals("prio")) {
                                    prio.add(objeto);
                                }
                            }
                        }
                        cascata.removeAll(prio);
                        delCascata.addAll(prio);
                    }
                }
                if (cascata.size() > 0) {
                    int aux = JOptionPane.showConfirmDialog(frame, "Existem " + cascata.size() + " componente(s) abaixo do objeto "
                            + o.getLabelWidget().getLabel() + "\nDeseja detelar todo(s)?",
                            "Aviso", JOptionPane.YES_NO_OPTION);

                    if (aux == JOptionPane.YES_OPTION) {
                        delCascata.addAll(cascata);
                        cascata.clear();
                    } else {
                        o.getChild().removeAll(cascata);
                        for (Objeto c : cascata) {
                            c.setNivel(0);
                            c.setLabel("");
                            if (c instanceof Qdisc) {
                                ((Qdisc) c).setHandle("");
                            } else if (c instanceof Classe) {
                                ((Classe) c).setClassid("");
                            }
                            c.getParent().getChild().remove(c);
                        }
                        for (Objeto c : delCascata) {
                            c.setNivel(0);
                        }
                        delCascata.clear();
                        cascata.clear();
                    }
                }
            }

            for (Objeto o : delCascata) {
                ObjetoNode on = (ObjetoNode) findObject(o);
                delNode.add(on);
            }
        }

        for (ObjetoNode o : delNode) {
            for (String s : getEdges()) {
                if ((getEdgeSource(s) == o) || (getEdgeTarget(s) == o)) {
                    delEdge.add(s);
                }
            }
        }

        for (String s : delEdge) {
            Objeto source = (Objeto) findWidget(getEdgeSource(s));
            Objeto target = (Objeto) findWidget(getEdgeTarget(s));
            if ((source instanceof Classe) && (target instanceof Qdisc)) {
                if (((Classe) source).getQdisc() != null) {
                    ((Classe) source).setQdisc(null);
                }
            } else if ((source instanceof ClassFul) && (target instanceof Classe)) {
                ((ClassFul) source).getClasses().remove(target);
                ((ClassFul) source).getClasses().removeAll(target.getChild());
            } else if ((source instanceof Classe) && (target instanceof Classe)) {
                ClassFul cf = getClassFul(source);
                if (cf != null) {
                    cf.getClasses().remove(target);
                }
            }
            source.getChild().remove(target);
            target.setParent((Objeto) null);
        }


        // Deleta as conexões selecionas e verifica se o usuário deseja deletar 
        // os objetos que estão abaixo de nível de hierarquia
        for (String s : delEdge) {
            ObjetoNode target = getEdgeTarget(s);
            if ((delNode.contains(getEdgeTarget(s)) == false) || (reconnect)) {
                Objeto o = (Objeto) findWidget(target);
                Objeto source = (Objeto) findWidget(getEdgeSource(s));
                Set<Objeto> cascata = new HashSet<Objeto>();
                cascata.addAll(getChildrenObjeto(o));
                cascata.add(o);
                if (o instanceof ClassFul) {
                    if (((ClassFul) o).getQdisc().equals("prio")) {
                        Set<Objeto> prio = new HashSet<Objeto>();
                        for (Objeto objeto : cascata) {
                            if (objeto instanceof Classe) {
                                if (((Classe) objeto).getNomClass().equals("prio")) {
                                    prio.add(objeto);
                                }
                            }
                        }
                        cascata.removeAll(prio);
                        delCascata.addAll(prio);
                    }
                }
                if (cascata.size() > 0) {
                    int aux = JOptionPane.showConfirmDialog(frame, "Deseja deletar o obeto " + o.getLabelWidget().getLabel()
                            + " e o(s) " + (cascata.size() - 1) + " componente(s) abaixo dele?",
                            "Aviso", JOptionPane.YES_NO_OPTION);
                    if (aux == JOptionPane.YES_OPTION) {
                        delCascata.addAll(cascata);
                        cascata.clear();
                    } else {
                        source.getChild().removeAll(cascata);
                        for (Objeto c : cascata) {
                            c.setNivel(0);
                            c.setLabel("");
                            if (c instanceof Qdisc) {
                                ((Qdisc) c).setHandle("");
                            } else if (c instanceof Classe) {
                                ((Classe) c).setClassid("");
                            }
                        }
                        for (Objeto c : delCascata) {
                            c.setNivel(0);
                        }
                        cascata.clear();
                        delCascata.clear();
                    }
                }

                for (Objeto del : delCascata) {
                    ObjetoNode on = (ObjetoNode) findObject(del);
                    delNode.add(on);
                }
            }
            if (reconnect == false) {
                removeEdge(s);
            }
        }

        // Deleta os objetos
        for (ObjetoNode on : delNode) {
            Objeto o = (Objeto) findWidget(on);
            niveis = niveis + o.getNivel();
            removeNode(on);
            nodeIDcounter--;
        }

        // Atualiza os niveis e os labels
        niveis = niveis + 0;
        updateLabel(niveis);
        validate();
        repaint();
    }

    /**
     * Função que retorna todos os "filhos" do objeto em questão
     *
     * @param o objeto pai
     * @return retorna o arraylist com todos os "filhos" do objeto pai
     */
    public ArrayList<Objeto> getChildrenObjeto(Objeto o) {
        ArrayList<Objeto> auxCascata = new ArrayList<Objeto>();

        auxCascata.addAll(o.getChild());
        int i = 0;
        while (i < auxCascata.size()) {
            Objeto objeto = auxCascata.get(i);
            auxCascata.addAll(objeto.getChild());
            i++;
        }

        return auxCascata;
    }

    /**
     * Função responsável por atualizar o nível e o label de todos os objetos
     * filhos de um objeto pai.
     *
     * @param o objeto pai
     */
    public void updateChildren(Objeto o) {
        ArrayList<Objeto> auxCascata = new ArrayList<Objeto>();

        if (o.getChild().size() > 0) {
            auxCascata.add(o);
        }
        int i = 0;
        while (i < auxCascata.size()) {
            for (Objeto child : o.getChild()) {
                child.setNivel(o.getNivel() + 1);
                updateLabel("" + child.getNivel());
                if (child.getChild().size() > 0) {
                    if (auxCascata.contains(child) == false) {
                        auxCascata.add(child);
                    }
                }
            }
            i++;
            if (i < auxCascata.size()) {
                o = auxCascata.get(i);
            }
        }

        validate();
        repaint();
    }

    /**
     * Função responsável por atualizar os labels dos objetos.
     *
     * @param niveis nível de hierarquia que deseja-se atualizar
     */
    public void updateLabel(String niveis) {
        String label = "";
        for (ObjetoNode on : getNodes()) {
            Objeto o = (Objeto) findWidget(on);
            if (niveis.contains("" + o.getNivel())) {
                if (o instanceof Qdisc) {
                    label = defineLabel(o.getNivel(), o.getParentString(), o);
                    o.setLabel(label);
                    ((Qdisc) o).setHandle(label);
                } else if (o instanceof Classe) {
                    if (!"prio".equals(((Classe) o).getNomClass())) {
                        if (!"".equals(o.getParentString())) {
                            label = defineLabel(o.getNivel(), o.getParentString(), o);
                            o.setLabel(label);
                            ((Classe) o).setClassid(label);
                        }
                    } else {
                        label = o.getParentString() + ((Classe) o).getBand();
                        o.setLabel(label);
                        ((Classe) o).setClassid(label);
                    }
                }

                // Faz com que atualize também os filhos desse objeto
                for (Objeto child : getChildrenObjeto(o)) {
                    if (child instanceof Qdisc) {
                        label = defineLabel(child.getNivel(), child.getParentString(), child);
                        child.setLabel(label);
                        ((Qdisc) child).setHandle(label);
                    } else if (child instanceof Classe) {
                        if (!"prio".equals(((Classe) child).getNomClass())) {
                            if (!"".equals(child.getParentString())) {
                                label = defineLabel(child.getNivel(), child.getParentString(), child);
                                child.setLabel(label);
                                ((Classe) child).setClassid(label);
                            }
                        } else {
                            label = child.getParentString() + ((Classe) child).getBand();
                            child.setLabel(label);
                            ((Classe) child).setClassid(label);
                        }
                    }
                }
            }
        }
    }

    /**
     * Função que serve para retornar uma conexão (edge)
     *
     * @param source objeto que é origem desta conexão
     * @param target objeto que é target desta conexão
     * @return retorna a conexão se existir.
     */
    public String getEdge(Objeto source, Objeto target) {
        for (String s : getEdges()) {
            ObjetoNode oSource = (ObjetoNode) findObject(source);
            ObjetoNode oTarget = (ObjetoNode) findObject(target);
            if ((getEdgeSource(s) == oSource) && (getEdgeTarget(s) == oTarget)) {
                return s;
            }
        }
        return null;
    }

    /**
     * Função responsável por definir o label dos objetos quando conectado a um
     * objeto pai.
     *
     * @param n nível do objeto
     * @param label label do objeto pai
     * @param o objeto pai
     * @return retorna o label
     */
    public String defineLabel(int n, String label, Objeto o) {
        String sufixo;
        switch (n) {
            case 0:
                return "";
            case 1:
                label = (contNivel(n, o) + 1) + ":";
                break;
            case 2:
                label = label + (contNivel(n, o) + 1);
                break;
            case 3:
                label = label + contNivel(n, o);
                break;
            case 4:
                sufixo = getSufixo(label);
                if (!"".equals(sufixo)) {
                    label = sufixo + ":";
                } else {
                    label = label + (contNivel(n, o) + 1);
                }
                break;
            case 5:
                label = label + (contNivel(n, o) + 1);
                break;
            case 6:
                label = label + contNivel(n, o);
                break;
            case 7:
                sufixo = getSufixo(label);
                if (!"".equals(sufixo)) {
                    label = sufixo + "0:";
                } else {
                    label = label.substring(label.length(), label.length() - 1) + "0:";
                }
                break;
            default:
                return "";
        }

        // Quando for uma Qdisc, verifica de já não existe um qdisc com esse label
        if ((o instanceof Qdisc) && (o.getImagemNome() != "qdisc.png")) {
            boolean repet = true, labelQdisc = true, existeLabel = false;
            int cont = 0;

            while (repet) {
                for (ObjetoNode on : getNodes()) {
                    Objeto objeto = (Objeto) findWidget(on);
                    if (objeto.getLabelWidget().getLabel() != null) {
                        if (objeto.getLabelWidget().getLabel().equals(label)
                                && o != objeto) {
                            existeLabel = true;
                            break;
                        }
                    }
                }

                for (int i = 0; i < label.length(); i++) {
                    int ascii = (int) label.charAt(i);
                    if (ascii == 58) {
                        if (i == label.length() - 1) {
                            if (!existeLabel) {
                                return label;
                            }
                        } else {
                            labelQdisc = false;
                            break;
                        }
                    }
                }

                if ((!existeLabel) && (labelQdisc)) {
                    repet = false;
                } else {
                    label = o.getParentString();
                    String novoLabel = "";
                    for (int i = 0; i < label.length(); i++) {
                        int ascii = (int) label.charAt(i);
                        if (ascii != 58) {
                            novoLabel = novoLabel + label.charAt(i);
                        }
                    }
                    if (o.getParent() != null) {
                        if (o.getParent() instanceof Qdisc) {
                            label = novoLabel + cont + ":";
                        } else {
                            label = novoLabel + ":";
                        }
                    }
                }
                cont++;
            }
        }
        return label;
    }

    /**
     * Função que conta quantos objetos tem em um determinado nível
     *
     * @param n nivel
     * @param o objeto
     * @return retorna a quantidade de objetos.
     */
    public int contNivel(int n, Objeto o) {
        int i = 0;

        if (o.getParent() != null) {
            for (Objeto ob : o.getParent().getChild()) {
                if (ob.getNivel() == n) {
                    if (ob != o) {
                        if (ob.getParent() != null) {
                            if (ob.getParent() == o.getParent()) {
                                i++;
                            }
                        } else {
                            i++;
                        }
                    } else {
                        return i;
                    }
                }
            }
        } else {
            for (ObjetoNode on : getNodes()) {
                Objeto ob = (Objeto) findWidget(on);
                if (ob.getNivel() == n) {
                    if (ob != o) {
                        i++;
                    } else {
                        return i;
                    }
                }
            }
        }
        return i;
    }

    /**
     * Serve para pegar o sufixo de um label. Ou seja, o que vier antes de ":"
     * Exemplo: Label igual 1:10. Retorna 1
     *
     * @param s label
     * @return retorna o sufixo do label
     */
    public String getSufixo(String s) {
        for (int i = 0; i < s.length(); i++) {
            int ascii = (int) s.charAt(i);
            if (ascii == 58) {
                return s.substring(i + 1);
            }
        }
        return s;
    }

    /**
     * Função que localiza a Qdisc ClassFul que está acima do objeto.
     *
     * @param o obejto em questão
     * @return retorna uma Qdisc ClassFul
     */
    public ClassFul getClassFul(Objeto o) {
        for (ObjetoNode on : getNodes()) {
            Objeto objeto = (Objeto) findWidget(on);
            if (objeto instanceof ClassFul) {
                ClassFul cf = (ClassFul) objeto;
                for (Classe c : cf.getClasses()) {
                    if (c.equals(o)) {
                        return cf;
                    }
                }
            }
        }
        return null;
    }

    /**
     * Localiza o objeto pai a partir do label.
     *
     * @param id label do objeto pai
     * @return retorna o objeto pai.
     */
    public Objeto getParent(String id) {
        for (ObjetoNode on : getNodes()) {
            Objeto o = (Objeto) findWidget(on);
            if (o instanceof Qdisc) {
                Qdisc q = (Qdisc) o;
                if (q.getHandle() != null) {
                    if (q.getHandle().equals(id)) {
                        return q;
                    }
                }
            } else if (o instanceof Classe) {
                Classe c = (Classe) o;
                if (c.getClassid() != null) {
                    if (c.getClassid().equals(id)) {
                        return c;
                    }
                }
            }
        }
        return null;
    }

    /**
     * Esta função calcula o quanto de taixa de transmissão de uma disciplina de
     * fila HTB ainda está dísponível.
     *
     * @param pai Classe pai
     * @param c classe filha
     * @return retorna a taixa de transmmissão disponível
     */
    public int calcRateDisp(Classe pai, Classe c) {
        int maxRate = pai.getRate();
        int soma = 0;

        for (ObjetoNode on : getNodes()) {
            Objeto o = (Objeto) findWidget(on);
            if (o != c) {
                if (o instanceof Classe) {
                    if (((Classe) o).getParent() != null) {
                        if (((Classe) o).getParent().equals(pai)) {
                            soma = soma + ((Classe) o).getRate();
                        }
                    }
                }
            }
        }
        return maxRate - soma;
    }

    /**
     * Está função verifica se é possível criar a conexão entre dois objetos.
     *
     * @param source objeto de origem.
     * @param target objeto de destino.
     * @return
     */
    public boolean isCreateConnection(Objeto source, Objeto target) {
        if ((source.getImagemNome().equals("qdisc.png")) && (target.getImagemNome().equals("class_sub.png"))
                || (source.getImagemNome().contains("class")) && (target.getImagemNome().equals("qdisc.png"))
                || (source.getImagemNome().equals("qdisc_sub.png")) && (target.getImagemNome().equals("class_sub.png"))
                || (source.getImagemNome().equals("qdisc_sub.png")) && (target.getImagemNome().equals("qdisc.png"))
                || (source instanceof ClassLess) && (target instanceof ClassLess)
                || (source.getImagemNome().equals("qdisc_sub.png")) && (target.getImagemNome().equals("qdisc_sub.png"))
                || (source.getImagemNome().equals("qdisc.png")) && (target.getImagemNome().equals("qdisc.png"))) {
            return false;
        }
        return true;
    }

    /**
     * Verifica se já existe a conexão
     *
     * @param source objeto de origem
     * @param target objeto de destini
     * @return true se existe. False se não existe.
     */
    public boolean isExisteEdge(Objeto source, Objeto target) {
        for (String s : getEdges()) {
            if (s.equals(getEdge(source, target))) {
                return true;
            }
        }
        return false;
    }

    /**
     * Função que exporta o ambiente de trabalho em uma imagem .PNG
     */
    public void exportToPng() {

        setBackground(Color.WHITE);
        validate();

        //create buffered image
        BufferedImage bi = new BufferedImage(this.getBounds().width, this.getBounds().height, BufferedImage.TYPE_4BYTE_ABGR);
        Graphics2D graphics = bi.createGraphics();
        this.paint(graphics);
        graphics.dispose();

        JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle("Exportar imagem");
        chooser.setDialogType(JFileChooser.SAVE_DIALOG);
        chooser.setMultiSelectionEnabled(false);
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
        chooser.setEnabled(true);

        if ((lastLocate != null) || (lastFile != null)) {
            if (!"".equals(lastLocate)) {
                chooser.setCurrentDirectory(new File(lastLocate));
            }
            if (!"".equals(lastFile)) {
                chooser.setSelectedFile(new File(lastFile + ".png"));
            }
        }

        chooser.setFileFilter(new FileFilter() {
            public boolean accept(File file) {
                if (file.isDirectory()) {
                    return true;
                }
                return file.getName().toLowerCase().endsWith(".png"); // NOI18N
            }

            public String getDescription() {
                return "Imagem PNG"; // NOI18N
            }
        });
        if (chooser.showSaveDialog(getView()) != JFileChooser.APPROVE_OPTION) {
            if (frame.getjTBgrid().isSelected()) {
                initGrids();
            }
            return;
        }

        File file = chooser.getSelectedFile();
        if (!file.getName().toLowerCase().endsWith(".png")) // NOI18N
        {
            file = new File(file.getParentFile(), file.getName() + ".png"); // NOI18N
            lastFile = file.getName();
        } else {
            lastFile = file.getName().substring(0, file.getName().length() - 4);
        }

        if (file.exists()) {
            int yes = JOptionPane.showConfirmDialog(getView(), "Arquivo já existente, deseja subistituir?",
                    "Arquivo existente", JOptionPane.YES_NO_OPTION);
            if (yes != 0) {
                lastFile = "";
                return;
            }
        }

        try {
            lastLocate = chooser.getCurrentDirectory().getPath();
            ImageIO.write(bi, "png", file); // NOI18N
            if (frame.getjTBgrid().isSelected()) {
                initGrids();
            }
        } catch (IOException e) {
            ErrorManager.getDefault().notify(e);
        }
    }

    /**
     * Carrega um projeto salvo
     *
     * @return
     */
    public boolean load() {
        JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle("Abrir projeto");
        chooser.setMultiSelectionEnabled(false);
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

        if ((lastLocate != null) || (lastFile != null)) {
            if (!"".equals(lastLocate)) {
                chooser.setCurrentDirectory(new File(lastLocate));
            }
            if (!"".equals(lastFile)) {
                chooser.setSelectedFile(new File(lastFile + ".xml"));
            }
        }

        FileFilter xml = new FileNameExtensionFilter("Documento XML", "xml");
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setFileFilter(xml);

        if (chooser.showOpenDialog(getView()) == JFileChooser.APPROVE_OPTION) {
            for (String edge : new ArrayList<String>(getEdges())) {
                removeEdge(edge);
            }
            for (ObjetoNode node : new ArrayList<ObjetoNode>(getNodes())) {
                removeNode(node);
            }
            mainLayer.removeChildren();
            notas.clear();
            frame.clearFiltros();
            filtros.clear();
            arquivo = chooser.getSelectedFile();
            try {
                lastLocate = chooser.getCurrentDirectory().getPath();
                lastFile = chooser.getSelectedFile().getName();
                lastFile = lastFile.substring(0, lastFile.length() - 4);
                VisualSerializer.deserialize(this, chooser.getSelectedFile());
                return true;
            } catch (Exception ex) {
                for (String edge : new ArrayList<String>(getEdges())) {
                    removeEdge(edge);
                }
                for (ObjetoNode node : new ArrayList<ObjetoNode>(getNodes())) {
                    removeNode(node);
                }
                mainLayer.removeChildren();
                notas.clear();
                frame.clearFiltros();
                filtros.clear();
                JOptionPane.showMessageDialog(frame, "Arquivo " + arquivo.getName() + " está corrompido!",
                        "Erro", JOptionPane.ERROR_MESSAGE);
                return false;
            }
        }
        return false;
    }

    /**
     * Opção salvar um projeto como.
     */
    public void saveAs() {
        JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle("Salvar projeto como");
        chooser.setMultiSelectionEnabled(false);
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

        FileFilter xml = new FileNameExtensionFilter("Documento XML", "xml");
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setFileFilter(xml);

        if ((lastLocate != null) || (lastFile != null)) {
            if (!"".equals(lastLocate)) {
                chooser.setCurrentDirectory(new File(lastLocate));
            }
            if (!"".equals(lastFile)) {
                chooser.setSelectedFile(new File(lastFile + ".xml"));
            }
        }

        if (chooser.showSaveDialog(getView()) == JFileChooser.APPROVE_OPTION) {
            try {
                File f = chooser.getSelectedFile();
                if (f.exists()) {
                    int yes = JOptionPane.showConfirmDialog(getView(), "Arquivo já existente, deseja subistituir?",
                            "Arquivo existente", JOptionPane.YES_NO_OPTION);
                    if (yes != 0) {
                        return;
                    }
                }
                if (f.getUsableSpace() > f.getTotalSpace()) {
                    JOptionPane.showMessageDialog(frame, "Não há mais espaço livre em seu HD.",
                            "Erro", JOptionPane.ERROR_MESSAGE);
                    return;
                } else {

                    String arq = chooser.getSelectedFile().toString();
                    if ("xml".equals(arq.substring(arq.length() - 3))) {
                        arquivo = new File("" + chooser.getSelectedFile());
                        lastFile = arq.substring(0, arq.length() - 4);
                    } else {
                        arquivo = new File(chooser.getSelectedFile() + ".xml");
                        lastFile = arq;

                    }
                    lastLocate = chooser.getCurrentDirectory().getPath();
                    VisualSerializer.serialize(this, arquivo);
                }
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(frame, "Você não tem permissão para criar o arquivo " + arquivo,
                        "Erro", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    /**
     * Opção salvar projeto.
     */
    public void save() {
        try {
            VisualSerializer.serialize(this, arquivo);
        } catch (Exception ex) {
            JOptionPane.showMessageDialog(frame, "Você não tem permissão para criar o arquivo " + arquivo,
                    "Erro", JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Opção de gerar o shell script
     */
    public void geraSh() {
        JFileChooser chooser = new JFileChooser();
        chooser.setDialogTitle("Gerar shell script");
        chooser.setMultiSelectionEnabled(false);
        chooser.setFileSelectionMode(JFileChooser.FILES_ONLY);

        FileFilter sh = new FileNameExtensionFilter("Shell Script", "sh");
        chooser.setAcceptAllFileFilterUsed(false);
        chooser.setFileFilter(sh);

        if ((lastLocate != null) || (lastFile != null)) {
            if (!"".equals(lastLocate)) {
                chooser.setCurrentDirectory(new File(lastLocate));
            }
            if (!"".equals(lastFile)) {
                chooser.setSelectedFile(new File(lastFile + ".sh"));
            }
        }

        if (chooser.showSaveDialog(getView()) == JFileChooser.APPROVE_OPTION) {
            File f = chooser.getSelectedFile();
            if (f.exists()) {
                int yes = JOptionPane.showConfirmDialog(getView(), "Arquivo já existente, deseja subistituir?",
                        "Arquivo existente", JOptionPane.YES_NO_OPTION);
                if (yes != 0) {
                    return;
                }
            }
            if (f.getUsableSpace() > f.getTotalSpace()) {
                JOptionPane.showMessageDialog(frame, "Não há mais espaço livre em seu HD.",
                        "Erro", JOptionPane.ERROR_MESSAGE);
                return;
            } else {
                try {
                    String arq = chooser.getSelectedFile().toString();
                    lastLocate = chooser.getCurrentDirectory().getPath();
                    if ("sh".equals(arq.substring(arq.length() - 2))) {
                        writeSh(chooser.getSelectedFile().toString());
                        lastFile = arq.substring(0, arq.length() - 3);
                    } else {
                        lastFile = arq;
                        writeSh(chooser.getSelectedFile() + ".sh");
                    }
                } catch (Exception e) {
                    JOptionPane.showMessageDialog(frame, "Você não tem permissão para criar o arquivo " + arquivo,
                            "Erro", JOptionPane.ERROR_MESSAGE);
                }

            }
        }
    }

    /**
     * Função que gera um shell script com os respectivos comandos TC.
     *
     * @param f arquivo que será salvo
     */
    public void writeSh(String f) {
        ArrayList<Objeto> ordenaNode = new ArrayList<Objeto>();
        ArrayList<Objeto> prio = new ArrayList<Objeto>();
        int nivelAnt = 0;

        // data do sistema
        DateFormat dateFormat = new SimpleDateFormat("dd/MM/yyyy HH:mm:ss");
        //get current date time with Date()
        Date date = new Date();

        try {
            // Create file 
            FileWriter fstream = new FileWriter(f);
            BufferedWriter out = new BufferedWriter(fstream);
            out.write("#!/bin/bash");
            out.write("\n\n");
            out.write("# Shell script gerado a partir do arquivo de configuração " + projeto);
            out.write("\n# em " + dateFormat.format(date));
            out.write("\n# atraves do software VisualTC.\n\n");
            out.write("# Deletando a qdisc root existente\n");
            out.write("tc qdisc del dev " + iface + " root 2>/dev/null");
            out.write("\n");
            for (ObjetoNode on : getNodes()) {
                ordenaNode.add((Objeto) findWidget(on));
            }

            ordenaPorLabel(ordenaNode);

            for (Objeto objeto : ordenaNode) {
                if (objeto instanceof Classe) {
                    if (((Classe) objeto).getNomClass().equals("prio")) {
                        prio.add(objeto);
                    }
                }
            }

            ordenaNode.removeAll(prio);
            for (Objeto objeto : ordenaNode) {
                if (objeto.getNivel() > 0) {
                    if (nivelAnt != objeto.getNivel()) {
                        out.write("\n");
                        out.write("# " + objeto.getNivel() + "o nivel da hierarquia\n");
                    }
                    out.write(objeto.geraComandosTC());
                    out.write("\n");
                }
                nivelAnt = objeto.getNivel();
            }

            if (filtros.size() > 0) {
                out.write("\n# Filtros\n");
                for (Filter filter : filtros) {
                    out.write(filter.geraComandosTC());
                    out.write("\n");
                }
            }

            //Close the output stream
            out.close();
        } catch (Exception e) {//Catch exception if any
            JOptionPane.showMessageDialog(frame, "Você não tem permissão para criar o arquivo " + arquivo,
                    "Erro", JOptionPane.ERROR_MESSAGE);
        }
    }

    /**
     * Opção de selecionar todos os objetos.
     */
    public void selectAll() {
        setSelectedObjects(getObjects());
        validate();
        repaint();
    }

    public void setNodeIDcounter(int nodeIDcounter) {
        this.nodeIDcounter = nodeIDcounter;
    }

    public void setEdgeIDcounter(int edgeIDcounter) {
        this.edgeIDcounter = edgeIDcounter;
    }

    public int getNodeIDcounter() {
        return nodeIDcounter;
    }

    public int getEdgeIDcounter() {
        return edgeIDcounter;
    }

    public VisualConnectProvider getSCP() {
        return SCP;
    }

    public VisualReconnectProvider getSRP() {
        return SRP;
    }

    public void setNovo(boolean novo) {
        this.novo = novo;
    }

    public VisualInterface getFrame() {
        return frame;
    }

    public ArrayList<Filter> getFiltros() {
        return filtros;
    }

    public void setFiltros(ArrayList<Filter> filtros) {
        this.filtros = filtros;
    }

    public void setProjeto(String projeto) {
        this.projeto = projeto;
    }

    public void setIface(String iface) {
        this.iface = iface;
    }

    public void setArquivo(File arquivo) {
        this.arquivo = arquivo;
    }

    public String getProjeto() {
        return projeto;
    }

    public String getIface() {
        return iface;
    }

    public File getArquivo() {
        return arquivo;
    }

    public ArrayList<ComponentWidget> getNotas() {
        return notas;
    }

    public LayerWidget getMainLayer() {
        return mainLayer;
    }

    /**
     * Função que habilita todas as ações necessárias.
     */
    public void setActions() {
        getActions().addAction(ActionFactory.createRectangularSelectAction(this, backgroundLayer));
        getActions().addAction(ActionFactory.createPopupMenuAction(new SceneMenu(this)));
        getActions().addAction(ActionFactory.createZoomAction());
        getActions().addAction(ActionFactory.createPanAction());
    }

    public void ordenaPorLabel(ArrayList<Objeto> o) {
        Collections.sort(o, new Comparator<Objeto>() {
            @Override
            public int compare(Objeto o1, Objeto o2) {
                String n1 = o1 != null ? Integer.toString(o1.getNivel()) : "";
                String n2 = o2 != null ? Integer.toString(o2.getNivel()) : "";
                return n1.compareTo(n2);
            }
        });
    }

    /**
     * Opção de copiar um objeto.
     */
    public void copyObjeto() {
        for (Object o : getSelectedObjects()) {
            if (isNode(o)) {
                m_O = (Objeto) findWidget(o);
                m_On = new ObjetoNode(((ObjetoNode) o).getNomImagem());
                m_On.setTipo(m_O.getClass().getSimpleName());
            }
        }
    }

    /**
     * Opção de colar um objeto.
     *
     * @param x posição x do objeto.
     * @param y posição y do objeto.
     */
    public void pasteObjeto(int x, int y) {
        String text = "";

        if ((m_O != null) && (m_On != null)) {
            ObjetoNode pasteNode = new ObjetoNode(m_On.getNomImagem());
            pasteNode.setTipo(m_On.getTipo());
            Random r = new Random();
            novo = false;
            Objeto paste = (Objeto) addNode(pasteNode);
            paste.setPreferredLocation(paste.convertLocalToScene(new Point(x, y)));
            paste.setNivel(0);
            if (paste instanceof ClassFul) {
                ((ClassFul) paste).setParent(((ClassFul) m_O).getParent());
                ((ClassFul) paste).setHandle(((ClassFul) m_O).getHandle());
                ((ClassFul) paste).setQdisc(((ClassFul) m_O).getQdisc());
                ((ClassFul) paste).setCdefault(((ClassFul) m_O).getCdefault());
                if (((ClassFul) paste).getQdisc().equals("prio")) {
                    for (int i = 1; i <= 3; i++) {
                        novo = true;
                        ObjetoNode on = new ObjetoNode("prio");
                        on.setPos(paste.getPreferredLocation());
                        Classe prio = new Classe(this);
                        prio = (Classe) addNode(on);
                        if (i == 1) {
                            x = on.getPos().x - 90;
                        } else if (i == 2) {
                            x = on.getPos().x;
                        } else {
                            x = on.getPos().x + 90;
                        }
                        y = (on.getPos().y + 120);
                        prio.setPreferredLocation(prio.convertLocalToScene(new Point(x, y)));
                        setNodeIDcounter(getNodeIDcounter() + 1);
                        getSceneAnimator().animatePreferredLocation(prio, new Point(x, y));
                        getSCP().createConnection(paste, prio);
                    }
                }
                text = "Qdisc\n" + ((ClassFul) paste).getQdisc().toUpperCase();
            } else if (paste instanceof ClassLess) {
                ((ClassLess) paste).setParent(((ClassLess) m_O).getParent());
                ((ClassLess) paste).setHandle(((ClassLess) m_O).getHandle());
                ((ClassLess) paste).setQdisc(((ClassLess) m_O).getQdisc());
                ((ClassLess) paste).setLimit(((ClassLess) m_O).getLimit());
                ((ClassLess) paste).setQuantum(((ClassLess) m_O).getQuantum());
                ((ClassLess) paste).setPertub(((ClassLess) m_O).getPertub());
                ((ClassLess) paste).setRate(((ClassLess) m_O).getRate());
                ((ClassLess) paste).setBurst(((ClassLess) m_O).getBurst());
                ((ClassLess) paste).setLatency(((ClassLess) m_O).getLatency());
                ((ClassLess) paste).setPeakrate(((ClassLess) m_O).getPeakrate());
                ((ClassLess) paste).setMtu(((ClassLess) m_O).getMtu());
                text = "Qdisc\n" + ((ClassLess) paste).getQdisc().toUpperCase();
            } else if (paste instanceof Classe) {
                ((Classe) paste).setParent(((Classe) m_O).getParent());
                ((Classe) paste).setClassid(((Classe) m_O).getClassid());
                ((Classe) paste).setNomClass(((Classe) m_O).getNomClass());
                ((Classe) paste).setRate(((Classe) m_O).getRate());
                ((Classe) paste).setCeil(((Classe) m_O).getCeil());
                ((Classe) paste).setBurst(((Classe) m_O).getBurst());
                ((Classe) paste).setCburst(((Classe) m_O).getCburst());
                ((Classe) paste).setPrio(((Classe) m_O).getPrio());
                text = "Classe\n" + ((Classe) paste).getNomClass().toUpperCase();
            }
            paste.setImage(insertTextImage(paste.getImageWidget().getImage(), text));
            nodeIDcounter++;
            validate();
            repaint();
        }
    }
}
